#ifdef NEC
/*
 * Copyright (c) 2001, 2002 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#ifndef __CONFIG_H__
#define __CONFIG_H__

/******************************************************************************/
/* Select Link Layer (Ethernet or PPP) */
#define	CONFIG_LINKLAYER		1					/* 1: Ethernet, 2: PPP */
/******************************************************************************/
/* Network parameter (LAN) */
#if (CONFIG_LINKLAYER == 1)
#define	CONFIG_DHCP				0					/* 0: OFF, 1: ON */
#define	CONFIG_MY_IPADDRESS		0x6400a8c0			/* for no DHCP */
#define	CONFIG_NETMASK			0x00ffffff			/* for no DHCP */
#define	CONFIG_GATEWAY			0xfe00a8c0			/* for no DHCP */
#define	CONFIG_DNS_SERVER1		0x0100a8c0			/* for no DHCP */
#define	CONFIG_DNS_SERVER2		0x00000000			/* for no DHCP */
#define	CONFIG_DOMAIN			"localdomain"		/* for no DHCP, MAX 64 characters */
#define	CONFIG_HOSTNAME			"tk850_jg2"			/* MAX 64 characters */
#endif

/******************************************************************************/
/* ---------- Memory options ---------- */
/* MEM_ALIGNMENT: should be set to the alignment of the CPU for which
   lwIP is compiled. 4 byte alignment -> define MEM_ALIGNMENT to 4, 2
   byte alignment -> define MEM_ALIGNMENT to 2. */
#define MEM_ALIGNMENT			4

/* MEM_SIZE: the size of the heap memory. If the application will send
a lot of data that needs to be copied, this should be set high. This 
must be less than 64000.*/
#define MEM_SIZE				5000

/* MEMP_NUM_PBUF: the number of memp struct pbufs. If the application
   sends a lot of data out of ROM (or other static memory), this
   should be set high. */
#define MEMP_NUM_PBUF			16

/* MEMP_NUM_RAW_PCB: the number of UDP protocol control blocks. One
   per active RAW "connection". */
#define MEMP_NUM_RAW_PCB		0

/* MEMP_NUM_UDP_PCB: the number of UDP protocol control blocks. One
   per active UDP "connection". */
#define MEMP_NUM_UDP_PCB		2

/* MEMP_NUM_TCP_PCB: the number of simulatenously active TCP
   connections. */
#define MEMP_NUM_TCP_PCB		20

/* MEMP_NUM_TCP_PCB_LISTEN: the number of listening TCP
   connections. */
#define MEMP_NUM_TCP_PCB_LISTEN	2

/* MEMP_NUM_TCP_SEG: the number of simultaneously queued TCP
   segments. */
#define MEMP_NUM_TCP_SEG		8

/* MEMP_NUM_SYS_TIMEOUT: the number of simulateously active
   timeouts. */
#define MEMP_NUM_SYS_TIMEOUT	10

/* The following four are used only with the sequential API and can be
   set to 0 if the application only will use the raw API. */
/* MEMP_NUM_NETBUF: the number of struct netbufs. */
#if	(CNET_NOSYS == 0)
#define MEMP_NUM_NETBUF			4
#else	/* CNET_NOSYS */
#define MEMP_NUM_NETBUF			0
#endif	/* CNET_NOSYS */

/* MEMP_NUM_NETCONN: the number of struct netconns. */
#if	(CNET_NOSYS == 0)
#define MEMP_NUM_NETCONN		10
#else	/* CNET_NOSYS */
#define MEMP_NUM_NETCONN		0
#endif	/* CNET_NOSYS */

/* MEMP_NUM_APIMSG: the number of struct api_msg, used for
   communication between the TCP/IP stack and the sequential
   programs. */
#if	(CNET_NOSYS == 0)
#define MEMP_NUM_API_MSG		4
#else	/* CNET_NOSYS */
#define MEMP_NUM_API_MSG		0
#endif	/* CNET_NOSYS */

/* MEMP_NUM_TCPIPMSG: the number of struct tcpip_msg, which is used
   for sequential API communication and incoming packets. Used in
   src/api/tcpip.c. */
#if	(CNET_NOSYS == 0)
#define MEMP_NUM_TCPIP_MSG		16
#else	/* CNET_NOSYS */
#define MEMP_NUM_TCPIP_MSG		0
#endif	/* CNET_NOSYS */

/* MALLOC_SIZE: the size that can be allocated with malloc. */
#define	MALLOC_SIZE				512

/******************************************************************************/
/* ---------- Pbuf options ---------- */
/* PBUF_POOL_SIZE: the number of buffers in the pbuf pool. */
#define PBUF_POOL_SIZE			18

/* PBUF_POOL_BUFSIZE: the size of each pbuf in the pbuf pool. */
#define PBUF_POOL_BUFSIZE		256

/* PBUF_LINK_HLEN: the number of bytes that should be allocated for a
   link level header. */
#define PBUF_LINK_HLEN			16

/******************************************************************************/
/* ---------- TCP options ---------- */
#define TCP_TTL					255

/* TCP Maximum segment size. */
#define TCP_MSS					1024

/* TCP sender buffer space (bytes). */
#define TCP_SND_BUF				2048

/* TCP sender buffer space (pbufs). This must be at least = 2 *
   TCP_SND_BUF/TCP_MSS for things to work. */
#define TCP_SND_QUEUELEN		16 * TCP_SND_BUF/TCP_MSS

/* TCP receive window. */
#define TCP_WND					1024

/* Maximum number of retransmissions of data segments. */
#define TCP_MAXRTX				12

/* Maximum number of retransmissions of SYN segments. */
#define TCP_SYNMAXRTX			4

/* TCP writable space (bytes). This must be less than or equal
   to TCP_SND_BUF. It is the amount of space which must be
   available in the tcp snd_buf for select to return writable */
#define	TCP_SNDLOWAT			TCP_SND_BUF/2

/******************************************************************************/
/* ---------- UDP options ---------- */
#define UDP_TTL					255

/******************************************************************************/
/* ---------- ICMP options ---------- */
#define ICMP_TTL				255

/******************************************************************************/
/* ---------- RAW options ---------- */
#define	CONFIG_RAW_IMPLEMENT	0
#define RAW_TTL					255

/******************************************************************************/
/* ---------- ARP options ---------- */
#if (CONFIG_LINKLAYER == 1)
#define	CONFIG_ETHARP_IMPLEMENT	1
#define ARP_TABLE_SIZE			3
#else
#define	CONFIG_ETHARP_IMPLEMENT	0
#define ARP_TABLE_SIZE			0
#endif

/******************************************************************************/
#if (CONFIG_LINKLAYER == 1)
#define CNET_PPP				0
#define CNET_FSDATA				1
#define CNET_PARAM				1
#define CNET_UARTSETUP			1
#define CNET_MAILSAMPLE			1
#define CNET_HTTPD				1
#define CNET_UART1				1
#define CNET_UART2				1
#define CNET_LANC				1
#elif (CONFIG_LINKLAYER == 2)
#define CNET_PPP				1
#define CNET_FSDATA				0
#define CNET_PARAM				0
#define CNET_UARTSETUP			0
#define CNET_MAILSAMPLE			0
#define CNET_HTTPD				0
#define CNET_UART1				1
#define CNET_UART2				0
#define CNET_LANC				0
#endif
/******************************************************************************/

#endif	/* __CONFIG_H__ */

#endif /* NEC */
