/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : main.c                                                        */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Main Routine Source Code for OS less                          */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#if (CONFIG_LINKLAYER == 1)
#include "lwip/netif.h"
#include "lwip/ip.h"
#include "lwip/ip_addr.h"
#include "api/cnet_init.h"
#include "api/cnet_systimer.h"
#include "api/cnet_single.h"
#include "api/cnet_dhcpc.h"
#include "api/cnet_httpd.h"
#include "cnet_param.h"
#include "cnet_lanc.h"
#include "cnet_uart.h"
#elif (CONFIG_LINKLAYER == 2)
#include "api/cnet_init.h"
#include "api/cnet_systimer.h"
#include "api/cnet_single.h"
#include "netif/ppp.h"
#include "cnet_uart.h"
#include "cnet_ppp.h"

static const struct ppp_config	ppp_config	= {
	uart1_recv,
	uart1_send,
	PPP_FSM_TIMEOUT,
	PPP_AUTH_TIMEOUT,
	PPP_LCP_ECHOINTERVAL
};

static struct ppp_addrs			ppp_addrs	= {0};
#endif	/* CONFIG_LINKLAYER */
/*----------------------------------------------------------------------------*/
#pragma interrupt	INTTQ0CC0	ms_counter
__interrupt void
ms_counter(void)
{
	inc_systimer();
}
/*----------------------------------------------------------------------------*/
#if CNET_LANC
#pragma interrupt	INTP4		lanc_interrupt
__interrupt void
lanc_interrupt(void)
{
	lan9115_interrupt();
}
#endif	/* CNET_LANC */
/*----------------------------------------------------------------------------*/
#if CNET_UART1
#pragma interrupt	INTUA1R		SerialRecvIntr1
__interrupt void
SerialRecvIntr1(void)
{
	uart1_recv_interrupt();
}
/*----------------------------------------------------------------------------*/
#pragma interrupt	INTUA1T		SerialSendIntr1
__interrupt void
SerialSendIntr1(void)
{
	uart1_send_interrupt();
}
#endif	/* CNET_UART1 */
/*----------------------------------------------------------------------------*/
#if CNET_UART2
#pragma interrupt	INTUA2R		SerialRecvIntr2
__interrupt void
SerialRecvIntr2(void)
{
	uart2_recv_interrupt();
}
/*----------------------------------------------------------------------------*/
#pragma interrupt	INTUA2T		SerialSendIntr2
__interrupt void
SerialSendIntr2(void)
{
	uart2_send_interrupt();
}
#endif	/* CNET_UART2 */
/*----------------------------------------------------------------------------*/
#if (CONFIG_LINKLAYER == 1)
static void
netif_setup(void)
{
	static struct netif	netif	= {0};
	struct ip_addr		ipaddr;
	struct ip_addr		netmask;
	struct ip_addr		gw;

	/* Setup monitor by UART I/O */
	uart_setupmon();

	if (get_param_integer(PARAM_DHCP)) {
		/* Use DHCP */
		IP4_ADDR(&gw, 0,0,0,0);
		IP4_ADDR(&ipaddr, 0,0,0,0);
		IP4_ADDR(&netmask, 0,0,0,0);

		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, lan9115if_init, ip_input);
		netif_set_default(&netif);

		if (cnet_dhcpstart(&netif) != 0) {
			/* When it doesn't go well, the use of DHCP is stopped. */
			cnet_dhcpstop(&netif);
			gw.addr = get_param_integer(PARAM_GATEWAY);
			ipaddr.addr = get_param_integer(PARAM_MYIP);
			netmask.addr = get_param_integer(PARAM_NETMASK);
			netif_set_addr(&netif, &ipaddr, &netmask, &gw);
			netif_set_up(&netif);
		}
	}
	else {
		/* Not use DHCP */
		gw.addr = get_param_integer(PARAM_GATEWAY);
		ipaddr.addr = get_param_integer(PARAM_MYIP);
		netmask.addr = get_param_integer(PARAM_NETMASK);

		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, lan9115if_init, ip_input);
		netif_set_default(&netif);
		netif_set_up(&netif);
	}

	/* UART output of setup result */
	uart_printinfo(&netif);
}
#endif	/* CONFIG_LINKLAYER */
/*----------------------------------------------------------------------------*/
int
main(int argc, char **argv)
{
#if (CONFIG_LINKLAYER == 1)
	/* Load the Parameter of FlashROM to RAM. */
	param_load();

	__EI();

	/* Network initialization */
	cnet_init();

	/* Setting of network interface */
	netif_setup();

	/* Start of HTTP server */
	cnet_httpdinit();

	while (1) {
		/* Execution of sample program of mail processing */
		mail_sample();

		msleep(1);
	}
#elif (CONFIG_LINKLAYER == 2)
	int				ret;

	__EI();

	/* Network initialization */
	cnet_init();

	/* PPP initialization */
	ret = ppp_init(&ppp_config);
	if (ret != 0) {
		goto end;
	}

	/* Start M1XTA01A */
	ret = m1xta01a_init();
	if (ret != 0) {
		goto end_m1xta01a;
	}

	/* Connect M1XTA01A */
	ret = m1xta01a_connect();
	if (ret != 0) {
		goto end_m1xta01a;
	}

	/* Something does */
	ret = ppp_getaddrs(&ppp_addrs);
	msleep(10000);

	/* Disconnect M1XTA01A */
	m1xta01a_disconnect();

end_m1xta01a:
	/* End M1XTA01A */
	m1xta01a_end();

end:
#endif	/* CONFIG_LINKLAYER */

	return 0;
}
/*----------------------------------------------------------------------------*/
