/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : lan9115_tk850_jg2.c                                           */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 OCT 04                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              LAN9115 Source Code for lwIP & TK-850/JG2+NET                 */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "lwip/sys.h"
#include "lwip/netif.h"
#include "lwip/inet.h"
#include "lwip/pbuf.h"
#include "netif/etharp.h"
#include "cnet_lanc.h"

#if CNET_LANC
/*----------------------------------------------------------------------------*/
/* Define those to better describe your network interface */
#define	IFNAME0						'e'
#define	IFNAME1						'n'

/* LAN9115 Base Address */
#define	LAN_BASEL					0x00100600
#define	LAN_BASEH					0x00100604
/*----------------------------------------------------------------------------*/
/* FIFO Port */
#define	RX_DATA_FIFO				0x0000
#define	TX_DATA_FIFO				0x0040
#define	RX_STATUS_FIFO				0x0080
#define	TX_STATUS_FIFO				0x0090

/* RX Status bits */
#define	RX_STATUS_L_ERROR_STATUS	0x8000
#define	RX_STATUS_H_ERROR_STATUS	0x0000
#define	RX_STATUS_L_PACKET_LENGTH	0x0000
#define	RX_STATUS_H_PACKET_LENGTH	0x3FFF

/* TX Command bits */
#define	TX_COMMAND_A_L				0x3000
#define	TX_COMMAND_A_H				0x0000
/*----------------------------------------------------------------------------*/
/* Control and Status Registers */
#define	ID_REV						0x00A0
#define	IRQ_CFG						0x00A8
#define	INT_STS						0x00B0
#define	INT_EN						0x00B8
#define	BYTE_TEST					0x00C8
#define	FIFO_INT					0x00D0
#define	RX_CFG						0x00D8
#define	TX_CFG						0x00E0
#define	HW_CFG						0x00E8
#define RX_DP_CTL					0x00F0
#define	RX_FIFO_INF					0x00F8
#define	TX_FIFO_INF					0x0100
#define	PMT_CTRL					0x0108
#define	GPIO_CFG					0x0110
#define	GPT_CFG						0x0118
#define	GPT_CNT						0x0120
#define	ENDIAN						0x0130
#define	FREE_RUN					0x0138
#define	RX_DROP						0x0140
#define	MAC_CSR_CMD					0x0148
#define	MAC_CSR_DATA				0x0150
#define	AFC_CFG						0x0158
#define	E2P_CMD						0x0160
#define	E2P_DATA					0x0168

/* IRQ_CFG bits */
#define	IRQ_CFG_L_DISABLE			0x0011
#define	IRQ_CFG_H_DISABLE			0x0000
#define	IRQ_CFG_L_ENABLE			0x0111
#define	IRQ_CFG_H_ENABLE			0x0000

/* INT_STS bits */
#define	INT_STS_L_RSFL				0x0008
#define	INT_STS_H_RSFL				0x0000
#define	INT_STS_L_RXE				0x4000
#define	INT_STS_H_RXE				0x0000
#define	INT_STS_L_PHY_INT			0x0000
#define	INT_STS_H_PHY_INT			0x0004
#define	INT_STS_L_SW_INT			0x0000
#define	INT_STS_H_SW_INT			0x8000

/* INT_EN bits */
#define	INT_EN_L_RSFL				0x0008
#define	INT_EN_H_RSFL				0x0000
#define	INT_EN_L_PHY_INT			0x0000
#define	INT_EN_H_PHY_INT			0x0004
#define	INT_EN_L_SW_INT				0x0000
#define	INT_EN_H_SW_INT				0x8000

/* TX_CFG bits */
#define	TX_CFG_L_TX_ON				0x0002
#define	TX_CFG_H_TX_ON				0x0000
#define	TX_CFG_L_TXSAO				0x0004
#define	TX_CFG_H_TXSAO				0x0000

/* HW_CFG bits */
#define	HW_CFG_L_SRST				0x0001
#define	HW_CFG_H_SRST				0x0000
#define	HW_CFG_L_SF					0x0000
#define	HW_CFG_H_SF					0x0010

/* RX_DP_CTL bits */
#define	RX_DP_CTL_L_RX_FFWD			0x0000
#define	RX_DP_CTL_H_RX_FFWD			0x8000

/* RX_FIFO_INF bits */
#define	RX_FIFO_INF_L_RXSUSED		0x0000
#define	RX_FIFO_INF_H_RXSUSED		0x00FF

/* TX_FIFO_INF bits */
#define	TX_FIFO_INF_L_TXSUSED		0x0000
#define	TX_FIFO_INF_H_TXSUSED		0x00FF

/* PMT_CTRL bits */
#define	PMT_CTRL_L_READY			0x0001
#define	PMT_CTRL_H_READY			0x0000
#define	PMT_CTRL_L_PHY_RST			0x0400
#define	PMT_CTRL_H_PHY_RST			0x0000

/* GPIO_CFG bits */
#define	GPIO_CFG_L_LED_EN			0x0000
#define	GPIO_CFG_H_LED_EN			0x7707

/* MAC_CSR_CMD bits */
#define	MAC_CSR_CMD_L_BUSY			0x0000
#define	MAC_CSR_CMD_H_BUSY			0x8000
#define	MAC_CSR_CMD_L_READ			0x0000
#define	MAC_CSR_CMD_H_READ			0xC000
#define	MAC_CSR_CMD_L_WRITE			0x0000
#define	MAC_CSR_CMD_H_WRITE			0x8000

/* E2P_CMD bits */
#define	E2P_CMD_L_BUSY				0x0000
#define	E2P_CMD_H_BUSY				0x8000
/*----------------------------------------------------------------------------*/
/* MAC CSR Address */
#define	MAC_CR						0x0001
#define	ADDRH						0x0002
#define	ADDRL						0x0003
#define	HASHH						0x0004
#define	HASHL						0x0005
#define	MII_ACC						0x0006
#define	MII_DATA					0x0007
#define	FLOW						0x0008
#define	VLAN1						0x0009
#define	VLAN2						0x000A
#define	WUFF						0x000B
#define	WUCSR						0x000C

/* MAC_CR bits */
#define	MAC_CR_L_RXEN				0x0004
#define	MAC_CR_H_RXEN				0x0000
#define	MAC_CR_L_TXEN				0x0008
#define	MAC_CR_H_TXEN				0x0000
#define	MAC_CR_L_MCPAS				0x0000
#define	MAC_CR_H_MCPAS				0x0008
#define	MAC_CR_L_FDPX				0x0000
#define	MAC_CR_H_FDPX				0x0010
#define	MAC_CR_L_RCVOWN				0x0000
#define	MAC_CR_H_RCVOWN				0x0080

/* MII_ACC bits */
#define	MII_ACC_L_BUSY				0x0001
#define	MII_ACC_H_BUSY				0x0000
#define	MII_ACC_L_READ				0x0801
#define	MII_ACC_H_READ				0x0000
#define	MII_ACC_L_WRITE				0x0803
#define	MII_ACC_H_WRITE				0x0000
/*----------------------------------------------------------------------------*/
/* PHY CSR Address */
#define	PHY_BCR						0x0000
#define	PHY_BSR						0x0040
#define	PHY_ID1						0x0080
#define	PHY_ID2						0x00C0
#define	PHY_ANEG_ADV				0x0100
#define	PHY_ANEG_LPA				0x0140
#define	PHY_ANEG_EXP				0x0180
#define	PHY_MODE_CSR				0x0440
#define	PHY_SP_MODES				0x0480
#define	PHY_SP_CSI					0x06C0
#define	PHY_INT_SRC					0x0740
#define	PHY_INT_MASK				0x0780
#define	PHY_SP_CSR					0x07C0

/* PHY_BCR bits */
#define	PHY_BCR_RESET				0x8000
#define	PHY_BCR_ANEG				0x1200

/* PHY_BSR bits */
#define	PHY_BSR_LINK				0x0004
#define	PHY_BSR_ANEG_COMPLETE		0x0020

/* PHY_ANEG_ADV bits */
#define	PHY_ANEG_ADV_SET			0x01E1

/* PHY_INT_SRC bits */
#define	PHY_INT_SRC_LINK_DOWN		0x0010
#define	PHY_INT_SRC_ANEG_COMPLETE	0x0040

/* PHY_SP_CSR bits */
#define	PHY_SP_CSR_10HDX			0x0004
#define	PHY_SP_CSR_10FDX			0x0014
#define	PHY_SP_CSR_100HDX			0x0008
#define	PHY_SP_CSR_100FDX			0x0018
#define	PHY_SP_CSR_SPEED			0x001C
/*----------------------------------------------------------------------------*/
#define	REG_L(a)	*((volatile u16_t *)(LAN_BASEL+(a)))
#define	REG_H(a)	*((volatile u16_t *)(LAN_BASEH+(a)))

#define	MAC_READ(a,lv,hv,w1,w2) \
	do{ \
		w1=REG_L(MAC_CSR_CMD); \
		w2=REG_H(MAC_CSR_CMD); \
	}while(w2&MAC_CSR_CMD_H_BUSY); \
	REG_L(MAC_CSR_CMD)=a; \
	REG_H(MAC_CSR_CMD)=MAC_CSR_CMD_H_READ; \
	w1=REG_L(BYTE_TEST); \
	w2=REG_H(BYTE_TEST); \
	do{ \
		w1=REG_L(MAC_CSR_CMD); \
		w2=REG_H(MAC_CSR_CMD); \
	}while(w2&MAC_CSR_CMD_H_BUSY); \
	lv=REG_L(MAC_CSR_DATA); \
	hv=REG_H(MAC_CSR_DATA)

#define	MAC_WRITE(a,lv,hv,w1,w2) \
	do{ \
		w1=REG_L(MAC_CSR_CMD); \
		w2=REG_H(MAC_CSR_CMD); \
	}while(w2&MAC_CSR_CMD_H_BUSY); \
	REG_L(MAC_CSR_DATA)=lv; \
	REG_H(MAC_CSR_DATA)=hv; \
	REG_L(MAC_CSR_CMD)=a; \
	REG_H(MAC_CSR_CMD)=MAC_CSR_CMD_H_WRITE; \
	w1=REG_L(BYTE_TEST); \
	w2=REG_H(BYTE_TEST); \
	do{ \
		w1=REG_L(MAC_CSR_CMD); \
		w2=REG_H(MAC_CSR_CMD); \
	}while(w2&MAC_CSR_CMD_H_BUSY)

#define	PHY_READ(a,v,w1,w2) \
	do{ \
		MAC_READ(MII_ACC,w1,w2,w1,w2); \
	}while(w1&MII_ACC_L_BUSY); \
	MAC_WRITE(MII_ACC,MII_ACC_L_READ|a,MII_ACC_H_READ,w1,w2); \
	do{ \
		MAC_READ(MII_ACC,w1,w2,w1,w2); \
	}while(w1&MII_ACC_L_BUSY); \
	MAC_READ(MII_DATA,v,w2,w1,w2)

#define	PHY_WRITE(a,v,w1,w2) \
	do{ \
		MAC_READ(MII_ACC,w1,w2,w1,w2); \
	}while(w1&MII_ACC_L_BUSY); \
	MAC_WRITE(MII_DATA,v,0x0000,w1,w2); \
	MAC_WRITE(MII_ACC,MII_ACC_L_WRITE|a,MII_ACC_H_WRITE,w1,w2); \
	do{ \
		MAC_READ(MII_ACC,w1,w2,w1,w2); \
	}while(w1&MII_ACC_L_BUSY)

#define	DUPLEX_SWITCH(lv,hv,w1,w2) \
	PHY_READ(PHY_SP_CSR,lv,w1,w2); \
	lv&=PHY_SP_CSR_SPEED; \
	if(lv==PHY_SP_CSR_10FDX||lv==PHY_SP_CSR_100FDX){ \
		MAC_READ(MAC_CR,lv,hv,w1,w2); \
		hv&=~MAC_CR_H_RCVOWN; \
		hv|=MAC_CR_H_FDPX; \
	} \
	else{ \
		MAC_READ(MAC_CR,lv,hv,w1,w2); \
		hv&=~MAC_CR_H_FDPX; \
		hv|=MAC_CR_H_RCVOWN; \
	} \
	MAC_WRITE(MAC_CR,lv,hv,w1,w2)
/*----------------------------------------------------------------------------*/
static struct lanc_state		lan9115_state	= {0};
#if	(CNET_NOSYS == 0)
static int						waitid			= 0;
#endif	/* CNET_NOSYS */

#pragma ioreg
/*----------------------------------------------------------------------------*/
/* initializes the LAN9115 chip */
static err_t
lan9115_init(struct netif *netif)
{
	u16_t	regl;
	u16_t	regh;
	u16_t	w1;
	u16_t	w2;

	/* Interrupt disable */
	PMK4 = 1;
	PIF4 = 0;

	/* Chip selection */
	PCT.6 = 0;

	/* Hardware reset */
	P9H.1 = 0;
	msleep(1);
	P9H.1 = 1;

	/* Device Ready check */
	do {
		msleep(1);
		regl = REG_L(PMT_CTRL);
		regh = REG_H(PMT_CTRL);
	} while (!(regl & PMT_CTRL_L_READY));

	/* Soft Reset */
	REG_L(HW_CFG) = HW_CFG_L_SRST;
	REG_H(HW_CFG) = HW_CFG_H_SRST;
	do {
		msleep(1);
		regl = REG_L(HW_CFG);
		regh = REG_H(HW_CFG);
	} while (regl & HW_CFG_L_SRST);

	/* Store and Forward */
	regl |= HW_CFG_L_SF;
	regh |= HW_CFG_H_SF;
	REG_L(HW_CFG) = regl;
	REG_H(HW_CFG) = regh;

	/* LED output */
	REG_L(GPIO_CFG) = GPIO_CFG_L_LED_EN;
	REG_H(GPIO_CFG) = GPIO_CFG_H_LED_EN;

	/* MAC address setting */
	do {
		regl = REG_L(E2P_CMD);
		regh = REG_H(E2P_CMD);
	} while (regh & E2P_CMD_H_BUSY);
	MAC_READ(ADDRH, regl, regh, w1, w2);
	netif->hwaddr[5] = regl >> 8 & 0xff;
	netif->hwaddr[4] = regl & 0xff;
	MAC_READ(ADDRL, regl, regh, w1, w2);
	netif->hwaddr[3] = regh >> 8 & 0xff;
	netif->hwaddr[2] = regh & 0xff;
	netif->hwaddr[1] = regl >> 8 & 0xff;
	netif->hwaddr[0] = regl & 0xff;

	/* PHY Reset */
//	REG_L(PMT_CTRL) = PMT_CTRL_L_PHY_RST;
//	REG_H(PMT_CTRL) = PMT_CTRL_H_PHY_RST;
//	do {
//		msleep(1);
//		regl = REG_L(PMT_CTRL);
//		regh = REG_H(PMT_CTRL);
//	} while (regl & PMT_CTRL_L_PHY_RST);

	/* PHY Soft Reset */
	PHY_WRITE(PHY_BCR, PHY_BCR_RESET, w1, w2);
	do {
		msleep(1);
		PHY_READ(PHY_BCR, regl, w1, w2);
	} while (regl & PHY_BCR_RESET);

	/* Auto-negotiation */
	PHY_WRITE(PHY_ANEG_ADV, PHY_ANEG_ADV_SET, w1, w2);
	PHY_WRITE(PHY_BCR, PHY_BCR_ANEG, w1, w2);
	regh = 20;
	do {
		msleep(500);
		PHY_READ(PHY_BSR, regl, w1, w2);
		if (regl & PHY_BSR_ANEG_COMPLETE) {
			break;
		}
		regh--;
	} while (regh > 0);

	/* Transmitter & Receiver Enable */
	regl = MAC_CR_L_RXEN | MAC_CR_L_TXEN | MAC_CR_L_MCPAS;
	regh = MAC_CR_H_RXEN | MAC_CR_H_TXEN | MAC_CR_H_MCPAS;
	MAC_WRITE(MAC_CR, regl, regh, w1, w2);
	REG_L(TX_CFG) = TX_CFG_L_TX_ON | TX_CFG_L_TXSAO;
	REG_H(TX_CFG) = TX_CFG_H_TX_ON | TX_CFG_H_TXSAO;

	/* Interrupt setting */
#if	(CNET_NOSYS == 0)
	PIC4 = 0x07;
#endif	/* CNET_NOSYS */
	regl = PHY_INT_SRC_LINK_DOWN | PHY_INT_SRC_ANEG_COMPLETE;
	PHY_WRITE(PHY_INT_MASK, regl, w1, w2);
	REG_L(IRQ_CFG) = IRQ_CFG_L_ENABLE;
	REG_H(IRQ_CFG) = IRQ_CFG_H_ENABLE;
	REG_L(INT_EN) = INT_EN_L_RSFL | INT_EN_L_PHY_INT;
	REG_H(INT_EN) = INT_EN_H_RSFL | INT_EN_H_PHY_INT;

	/* Link Check */
	PHY_READ(PHY_BSR, regl, w1, w2);
	PHY_READ(PHY_BSR, regl, w1, w2);
	if (regl & PHY_BSR_LINK) {
		DUPLEX_SWITCH(regl, regh, w1, w2);
		netif->flags |= NETIF_FLAG_LINK_UP;
	}

	return ERR_OK;
}
/*----------------------------------------------------------------------------*/
static err_t
lan9115_output(struct netif *netif, struct pbuf *p)
{
	union {
		u16_t	*u16;
		u8_t	*u8;
	} ptr;
	union {
		u16_t	u16[2];
		u8_t	u8[4];
	} data;
	u16_t		len;
	u16_t		idx;

	/* exit if link has failed */
	if (!(netif->flags & NETIF_FLAG_LINK_UP)) {
		return ERR_CONN;
	}

#if	(CNET_NOSYS == 0)
	sys_thread_disable_dispatch();
#endif	/* CNET_NOSYS */

	/* Read TX Status */
	data.u16[0] = REG_L(TX_FIFO_INF);
	data.u16[1] = REG_H(TX_FIFO_INF);
	while (data.u16[1] & TX_FIFO_INF_H_TXSUSED) {
		data.u16[0] = REG_L(TX_STATUS_FIFO);
		data.u16[1] = REG_H(TX_STATUS_FIFO);
		data.u16[0] = REG_L(TX_FIFO_INF);
		data.u16[1] = REG_H(TX_FIFO_INF);
	}

	/* Check TX Data Space */
	len = p->tot_len + 9;	/* Overhead 6 bytes + PAD (MAX) 3 bytes */
	while (data.u16[0] < len) {
		data.u16[0] = REG_L(TX_FIFO_INF);
		data.u16[1] = REG_H(TX_FIFO_INF);
	}

	/* TX Command A */
	REG_L(TX_DATA_FIFO) = TX_COMMAND_A_L | p->tot_len;
	REG_H(TX_DATA_FIFO) = TX_COMMAND_A_H;

	/* TX Command B */
	REG_L(TX_DATA_FIFO) = p->tot_len;
	REG_H(TX_DATA_FIFO) = p->tot_len;

	/* Write Packet */
	idx = 0;
	while (p != (struct pbuf *)0) {
		ptr.u16 = p->payload;
		len = p->len;
		if (idx > 0) {
			while (len > 0) {
				data.u8[idx] = *ptr.u8;
				ptr.u8++;
				len--;
				if (idx >= 3) {
					REG_L(TX_DATA_FIFO) = data.u16[0];
					REG_H(TX_DATA_FIFO) = data.u16[1];
					idx = 0;
					break;
				}
				idx++;
			}
		}
		while (len > 3) {
			REG_L(TX_DATA_FIFO) = *ptr.u16;
			ptr.u16++;
			REG_H(TX_DATA_FIFO) = *ptr.u16;
			ptr.u16++;
			len -= 4;
		}
		while (len > 0) {
			data.u8[idx] = *ptr.u8;
			ptr.u8++;
			idx++;
			len--;
		}
		p = p->next;
	}
	if (idx > 0) {
		while (idx < 4) {
			data.u8[idx] = 0x00;
			idx++;
		}
		REG_L(TX_DATA_FIFO) = data.u16[0];
		REG_H(TX_DATA_FIFO) = data.u16[1];
	}

#if	(CNET_NOSYS == 0)
	sys_thread_enable_dispatch();
#endif	/* CNET_NOSYS */

	return ERR_OK;
}
/*----------------------------------------------------------------------------*/
static err_t
lan9115if_output(struct netif *netif, struct pbuf *p, struct ip_addr *ipaddr)
{
	/* resolve hardware address, then send (or queue) packet */
	return etharp_output(netif, ipaddr, p);
}
/*----------------------------------------------------------------------------*/
static void
lan9115if_input(struct netif *netif)
{
	struct pbuf	*p	= (struct pbuf *)0;
	struct pbuf	*q	= (struct pbuf *)0;
	union {
		u16_t	*u16;
		u8_t	*u8;
	} ptr;
	union {
		u16_t	u16[2];
		u8_t	u8[4];
	} data;
	u16_t		len;
	u16_t		idx;

#if	(CNET_NOSYS == 0)
	sys_thread_disable_dispatch();
#endif	/* CNET_NOSYS */

	/* Read Status */
	data.u16[0] = REG_L(RX_STATUS_FIFO);
	data.u16[1] = REG_H(RX_STATUS_FIFO);
	len = data.u16[1] & RX_STATUS_H_PACKET_LENGTH;
	if ((data.u16[0] & RX_STATUS_L_ERROR_STATUS) == 0) {
		if (len > 4) {
			p = pbuf_alloc(PBUF_LINK, len - 4, PBUF_POOL);
		}
	}
	if (p == (struct pbuf *)0) {
		/* Error */
		len += 3;
		len /= 4;
		if (len < 3) {
			while (len > 0) {
				data.u16[0] = REG_L(RX_DATA_FIFO);
				data.u16[1] = REG_H(RX_DATA_FIFO);
				len--;
			}
		}
		else {
			REG_L(RX_DP_CTL) = RX_DP_CTL_L_RX_FFWD;
			REG_H(RX_DP_CTL) = RX_DP_CTL_H_RX_FFWD;
			do {
				data.u16[0] = REG_L(RX_DP_CTL);
				data.u16[1] = REG_H(RX_DP_CTL);
			} while (data.u16[1] & RX_DP_CTL_H_RX_FFWD);
		}

#if	(CNET_NOSYS == 0)
		sys_thread_enable_dispatch();
#endif	/* CNET_NOSYS */

		return;
	}

	/* Read Packet */
	idx = 0;
	for (q = p; q != (struct pbuf *)0; q = q->next) {
		ptr.u16 = q->payload;
		len = q->len;
		if (idx > 0) {
			while (len > 0) {
				*ptr.u8 = data.u8[idx];
				ptr.u8++;
				len--;
				if (idx >= 3) {
					idx = 0;
					break;
				}
				idx++;
			}
		}
		while (len > 3) {
			*ptr.u16 = REG_L(RX_DATA_FIFO);
			ptr.u16++;
			*ptr.u16 = REG_H(RX_DATA_FIFO);
			ptr.u16++;
			len -= 4;
		}
		if (len > 0) {
			data.u16[0] = REG_L(RX_DATA_FIFO);
			data.u16[1] = REG_H(RX_DATA_FIFO);
			do {
				*ptr.u8 = data.u8[idx];
				ptr.u8++;
				idx++;
				len--;
			} while (len > 0);
		}
	}
	/* Read FCS or PAD */
	data.u16[0] = REG_L(RX_DATA_FIFO);
	data.u16[1] = REG_H(RX_DATA_FIFO);

#if	(CNET_NOSYS == 0)
	sys_thread_enable_dispatch();
#endif	/* CNET_NOSYS */

	/* points to packet payload, which starts with an Ethernet header */
	idx = htons(((struct eth_hdr *)p->payload)->type);
	if (idx == ETHTYPE_IP) {
		/* IP packet */
		/* update ARP table */
		etharp_ip_input(netif, p);
		/* skip Ethernet header */
		pbuf_header(p, -14);
		/* pass to network layer */
		netif->input(p, netif);
	}
	else if (idx == ETHTYPE_ARP) {
		/* ARP packet */
		/* pass p to ARP module  */
		etharp_arp_input(netif, (struct eth_addr *)netif->hwaddr, p);
	}
	else {
		/* unsupported Ethernet packet type */
		/* free pbuf */
		pbuf_free(p);
		p = (struct pbuf *)0;
	}
}
/*----------------------------------------------------------------------------*/
static void
lan9115if_proc(struct netif *netif)
{
	u16_t	regl;
	u16_t	regh;
	u16_t	w1;
	u16_t	w2;

	REG_L(IRQ_CFG) = IRQ_CFG_L_DISABLE;
	REG_H(IRQ_CFG) = IRQ_CFG_H_DISABLE;

	while (1) {
		regl = REG_L(INT_STS);
		regh = REG_H(INT_STS);
		if (regh & INT_STS_H_PHY_INT) {
			/* PHY event */
			PHY_READ(PHY_INT_SRC, regl, w1, w2);
			PHY_READ(PHY_BSR, regl, w1, w2);
			PHY_READ(PHY_BSR, regl, w1, w2);
			if (regl & PHY_BSR_LINK) {
				/* Auto-Negotiation complete & Link Up */
				DUPLEX_SWITCH(regl, regh, w1, w2);
				netif->flags |= NETIF_FLAG_LINK_UP;
			}
			else {
				/* Link Down */
				netif->flags &= ~NETIF_FLAG_LINK_UP;
			}
		}
		else if (regl & INT_STS_L_RXE) {
			/* Receiver Error */
			REG_L(INT_STS) = INT_STS_L_RXE;
			REG_H(INT_STS) = INT_STS_H_RXE;
		}
		else if (regl & INT_STS_L_RSFL) {
			/* Receive packet */
			regl = REG_L(RX_FIFO_INF);
			regh = REG_H(RX_FIFO_INF);
			regh &= RX_FIFO_INF_H_RXSUSED;
			while (regh > 0) {
				lan9115if_input(netif);
				regl = REG_L(RX_FIFO_INF);
				regh = REG_H(RX_FIFO_INF);
				regh &= RX_FIFO_INF_H_RXSUSED;
			}
			REG_L(INT_STS) = INT_STS_L_RSFL;
			REG_H(INT_STS) = INT_STS_H_RSFL;
		}
		else if (regh & INT_STS_H_SW_INT) {
			/* Software Interrupt */
			regl = REG_L(INT_EN);
			regh = REG_H(INT_EN);
			regh &= ~INT_EN_H_SW_INT;
			REG_L(INT_EN) = regl;
			REG_H(INT_EN) = regh;
			REG_L(INT_STS) = INT_STS_L_SW_INT;
			REG_H(INT_STS) = INT_STS_H_SW_INT;
		}
		else {
			REG_L(IRQ_CFG) = IRQ_CFG_L_ENABLE;
			REG_H(IRQ_CFG) = IRQ_CFG_H_ENABLE;
			break;
		}
	}
}
/*----------------------------------------------------------------------------*/
err_t
lan9115if_init(struct netif *netif)
{
	if (netif == (struct netif *)0) {
		return ERR_ARG;
	}

	/* initialize lwip network interface */
	netif->name[0] = IFNAME0;
	netif->name[1] = IFNAME1;
	netif->output = lan9115if_output;
	netif->linkoutput = lan9115_output;
	netif->linkproc = lan9115if_proc;

	/* initialize lan9115 specific interface structure */
	netif->state = &lan9115_state;

	/* maximum transfer unit */
	netif->mtu = 1500;

	/* broadcast capability */
	netif->flags = NETIF_FLAG_BROADCAST;
  
	/* hardware address length */
	netif->hwaddr_len = 6;

	/* Task beginning */
#if	(CNET_NOSYS == 0)
	netif_thread_start((void *)netif);
#endif	/* CNET_NOSYS */

	etharp_timer_start();

	/* intialize the lan9115 chip */
	return lan9115_init(netif);
}
/*----------------------------------------------------------------------------*/
void
lan9115_interrupt(void)
{
#if	(CNET_NOSYS == 0)
	cnet_ready(&waitid);
#endif	/* CNET_NOSYS */
}
/*----------------------------------------------------------------------------*/
#if	(CNET_NOSYS == 0)
#pragma rtos_task lan9115if_thread
void
lan9115if_thread(int arg)
{
	cnet_wait_set(&waitid, 0);
	while (1) {
		cnet_wait(&waitid);
		lan9115if_proc((struct netif *)arg);
	}
}
#endif	/* CNET_NOSYS */
/*----------------------------------------------------------------------------*/
#endif	/* CNET_LANC */
