/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : param.c                                                       */
/* Type       : C Programming Language Source Code                            */
/* Version    : E1.00                                                         */
/* Date       : 2006 JUN 22                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              C-NET Parameter Source Code                                   */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "cnet_param.h"

#if CNET_PARAM
/*----------------------------------------------------------------------------*/
#define	CONFIG_SMTP_SERVER			"192.168.0.254"
#define	CONFIG_POP3_SERVER			"192.168.0.254"
#define	CONFIG_POP3_USER			"popuser"			/* MAX 40 characters */
#define	CONFIG_POP3_PASSWORD		"popuserpw"			/* MAX 40 characters */
#define	CONFIG_RECEIVE_MAIL_ADDRESS	"cnet@localhost"	/* MAX 64 characters */
#define	CONFIG_FORWARD_MAIL_ADDRESS	"root@localhost"	/* MAX 64 characters */
#define	CONFIG_MAIL_DELAY_TIME		30					/* seconds */

#define	MAX_PARAM_ID_INTEGER		PARAM_MAIL_DELAY
#define	MAX_PARAM_ID_STRING			PARAM_FWD_MAIL_ADDRESS

#define	FLASH_PARAM_BLOCK			12

struct flashdata {
	int		ipaddr;										/*   4 */
	int		netmask;									/*   8 */
	int		gwaddr;										/*  12 */
	int		dhcpflag;									/*  16 */
	int		maildelay;									/*  20 */
	char	hostname[MAX_HOSTNAME_LEN+1];				/*  85 */
	char	smtpserver[MAX_SMTPSERVER_LEN+1];			/* 150 */
	char	pop3server[MAX_POP3SERVER_LEN+1];			/* 215 */
	char	domainname[MAX_DOMAINNAME_LEN+1];			/* 280 */
	char	popuser[MAX_POPUSER_LEN+1];					/* 321 */
	char	poppass[MAX_POPPASS_LEN+1];					/* 362 */
	char	mailfrom[MAX_MAILFROM_LEN+1];				/* 427 */
	char	mailto[MAX_MAILTO_LEN+1];					/* 492 */
	char	dummy[4];									/* 496 */
};

/* Parameter area on RAM */
static struct flashdata			ramparam	= {0};

static const void				*param_p[]	= {
	&ramparam.ipaddr,
	&ramparam.netmask,
	&ramparam.gwaddr,
	&ramparam.dhcpflag,
	&ramparam.maildelay,
	ramparam.hostname,
	ramparam.smtpserver,
	ramparam.pop3server,
	ramparam.domainname,
	ramparam.popuser,
	ramparam.poppass,
	ramparam.mailfrom,
	ramparam.mailto
};

/* Initial value of parameter area on FlashROM */
#pragma section const "flash" begin
static const struct flashdata	flashparam	= {
	CONFIG_MY_IPADDRESS,
	CONFIG_NETMASK,
	CONFIG_GATEWAY,
	CONFIG_DHCP,
	CONFIG_MAIL_DELAY_TIME,
	CONFIG_HOSTNAME,
	CONFIG_SMTP_SERVER,
	CONFIG_POP3_SERVER,
	CONFIG_DOMAIN,
	CONFIG_POP3_USER,
	CONFIG_POP3_PASSWORD,
	CONFIG_RECEIVE_MAIL_ADDRESS,
	CONFIG_FORWARD_MAIL_ADDRESS,
	0
};
#pragma section const "flash" end

int flashrom_write(int block, int *in, int len);

/*----------------------------------------------------------------------------*/
int
get_param_integer(unsigned char id)
{
	if (id > MAX_PARAM_ID_INTEGER) {
		return 0;
	}

	return *((int *)param_p[id]);
}
/*----------------------------------------------------------------------------*/
char *
get_param_string(unsigned char id)
{
	if (id <= MAX_PARAM_ID_INTEGER) {
		return (char *)0;
	}
	if (id > MAX_PARAM_ID_STRING) {
		return (char *)0;
	}

	return (char *)param_p[id];
}
/*----------------------------------------------------------------------------*/
void
set_param_integer(unsigned char id, int value)
{
	if (id > MAX_PARAM_ID_INTEGER) {
		return;
	}

	*((int *)param_p[id]) = value;
}
/*----------------------------------------------------------------------------*/
void
set_param_string(unsigned char id, const char *value, unsigned short len)
{
	if (id <= MAX_PARAM_ID_INTEGER) {
		return;
	}
	if (id > MAX_PARAM_ID_STRING) {
		return;
	}

	strncpy(param_p[id], value, len);
}
/*----------------------------------------------------------------------------*/
void
param_load(void)
{
	ramparam = flashparam;
}
/*----------------------------------------------------------------------------*/
int
param_save(void)
{
	int		ret;
	int		i;

	for (i = 0; i < 2; i++) {
		ret = flashrom_write(FLASH_PARAM_BLOCK,
							(int *)&ramparam, sizeof(ramparam));
		if (ret == 0) {
			break;
		}
	}

	return ret;
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_PARAM */
