/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : flashrom_tk850_jg2.c                                          */
/* Type       : C Programming Language Source Code                            */
/* Version    : E1.00                                                         */
/* Date       : 2006 JUN 22                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Update flashrom Source Code for TK-850/JG2+NET                */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "flib.h"

static int					entry_ram[ENTRY_RAM_SIZE / sizeof(int)];
static struct FlashEnvData	envdata	= {20000000};					/* 20MHz */

#pragma ioreg
/*----------------------------------------------------------------------------*/
/* It is necessary to store the FlashROM access function on RAM. */
#pragma text "flash" flashrom_write

int
flashrom_write(int block, int *in, int len)
{
	int				*out;
	int				rval;
	unsigned short	mask[4];

	/* Other tasks and the interrupt processing on ROM must not move.
	   (interrupt/task switch prohibition) */
	sys_lock_cpu();

	mask[0] = IMR0;
	mask[1] = IMR1;
	mask[2] = IMR2;
	mask[3] = IMR3;
	IMR0 = 0xffff;
	IMR1 = 0xffff;
	IMR2 = 0xffff;
	IMR3 = 0xffff;
	
	/* FLMD0 to High */
	PM3L.2 = 0;
	P3L.2 = 1;

	/* The setting of the breakpoint etc.
	   do not go well when FLMD0 is made High. */

	/* Flash environment initialization */
	rval = FlashEnv(entry_ram, &envdata);
	if (rval != FE_OK) {
		rval |= 0x00010000;
		goto end;
	}

	/* Status check of terminal FLMD */
	rval = FlashFLMDCheck(entry_ram);
	if (rval != FE_OK) {
		rval |= 0x00020000;
		goto end;
	}

	/* Get address */
	if (block == 0) {
		out = (int *)0;
	}
	else {
		/* 4 + block <- 5 + (block - 1) */
		rval = FlashGetInfo(entry_ram, 4 + block);
		if (rval == 0) {
			rval = 0x00030000;
			goto end;
		}
		out = (int *)(rval + 1);
	}

	/* Delete block */
	rval = FlashBlockErase(entry_ram, block);
	if (rval != FE_OK) {
		rval |= 0x00040000;
		goto end;
	}

	/* Wait block deletion processing end */
	do {
		rval = FlashStatusCheck(entry_ram);
	} while (rval == FE_BUSY);
	if (rval != FE_OK) {
		rval |= 0x00050000;
		goto end;
	}

	len = (len + 15) / 16;
	len *= 4;
	while (len >= 64) {
		/* Flash writing */
		rval = FlashWordWrite(entry_ram, out, in, 64);
		if (rval != FE_OK) {
			rval |= 0x00060000;
			goto end;
		}

		/* Wait Flash writing processing end */
		do {
			rval = FlashStatusCheck(entry_ram);
		} while (rval == FE_BUSY);
		if (rval != FE_OK) {
			rval |= 0x00070000;
			goto end;
		}

		in += 64;
		out += 64;
		len -= 64;
	}
	if (len > 0) {
		/* Flash writing */
		rval = FlashWordWrite(entry_ram, out, in, len);
		if (rval != FE_OK) {
			rval |= 0x00060000;
			goto end;
		}

		/* Wait Flash writing processing end */
		do {
			rval = FlashStatusCheck(entry_ram);
		} while (rval == FE_BUSY);
		if (rval != FE_OK) {
			rval |= 0x00070000;
			goto end;
		}
	}

	/* Internal verify */
	rval = FlashBlockIVerify(entry_ram, block);
	if (rval != FE_OK) {
		rval |= 0x00080000;
		goto end;
	}

	/* Wait internal verify processing end */
	do {
		rval = FlashStatusCheck(entry_ram);
	} while (rval == FE_BUSY);
	if (rval != FE_OK) {
		rval |= 0x00090000;
	}

end:
	/* Flash environment end */
	FlashEnv(entry_ram, 0);

	/* FLMD0 to Low */
//	P3L.2 = 0;
	PM3L.2 = 1;

	/* It is possible to debug here. */
	IMR0 = mask[0];
	IMR1 = mask[1];
	IMR2 = mask[2];
	IMR3 = mask[3];

	/* Interrupt/task switch permission */
	sys_unlock_cpu();

	return rval;
}
/*----------------------------------------------------------------------------*/
