/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : httpdfunc.c                                                   */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              HTTP Server Contents Function Source Code                     */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "lwip/inet.h"
#include "lwip/def.h"
#include "arch/lib.h"
#include "api/cnet_httpd.h"
#include "cnet_param.h"
#include "cnet_led.h"
#include "cnet_uart.h"

#if CNET_HTTPD
/*----------------------------------------------------------------------------*/
#define	SERIAL_INIT		UART_INIT(2)
#define	SERIAL_SEND		UART_SEND(2)
#define MAXLINE         40
#define MAXPASSWDLEN	40
typedef	unsigned char	UART_CHAR;

static char				sendbuf[MAXLINE + 1]			= {0};
static char				pop3passwd[MAXPASSWDLEN + 1]	= {0};

/*----------------------------------------------------------------------------*/
void
cnet_httpd_datacmn(int id)
{
}
/*----------------------------------------------------------------------------*/
static void
send_ifmyaddr(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* IP Address */
	addr.s_addr = cnet_httpdgetipaddr(id);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_ifnetmask(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* Netmask */
	addr.s_addr = cnet_httpdgetnetmask(id);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_ifgateway(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* Gateway */
	addr.s_addr = cnet_httpdgetgwaddr(id);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_dhcp(int id, const char *name, const char *value)
{
	/* DHCP Function */
	if (get_param_integer(PARAM_DHCP)) {
		cnet_httpdsend(id, "ON", 2, 0);
	} else {
		cnet_httpdsend(id, "OFF", 3, 0);
	}
}
/*----------------------------------------------------------------------------*/
static void
send_myaddr(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* IP Address */
	addr.s_addr = get_param_integer(PARAM_MYIP);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_netmask(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* Netmask */
	addr.s_addr = get_param_integer(PARAM_NETMASK);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_gateway(int id, const char *name, const char *value)
{
	char			*p;
	struct in_addr	addr;

	/* Gateway */
	addr.s_addr = get_param_integer(PARAM_GATEWAY);
	p = inet_ntoa(addr);
	cnet_httpdsend(id, p, strlen(p), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_macaddr(int id, const char *name, const char *value)
{
	unsigned char	*addr;
	int				len;

	/* Mac address */
	addr = cnet_httpdgethwaddr(id);
	if (addr != (unsigned char *)0) {
		len = conv_macaddr(sendbuf, addr);
		cnet_httpdsend(id, sendbuf, len, 1);
	}
	return;
}
/*----------------------------------------------------------------------------*/
static void
send_smtpaddr(int id, const char *name, const char *value)
{
	/* SMTP Server IP Address */
	value = get_param_string(PARAM_SMTP_SERVER);
	cnet_httpdsend(id, value, strlen(value), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_mailfrom(int id, const char *name, const char *value)
{
	/* MAIL From: */
	value = get_param_string(PARAM_RCV_MAIL_ADDRESS);
	cnet_httpdsend(id, value, strlen(value), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_mailto(int id, const char *name, const char *value)
{
	/* MAIL To: */
	value = get_param_string(PARAM_FWD_MAIL_ADDRESS);
	cnet_httpdsend(id, value, strlen(value), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_pop3addr(int id, const char *name, const char *value)
{
	/* POP3 Server IP Address */
	value = get_param_string(PARAM_POP3_SERVER);
	cnet_httpdsend(id, value, strlen(value), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_popuser(int id, const char *name, const char *value)
{
	/* POP3 Server Account */
	value = get_param_string(PARAM_POP3_USER);
	cnet_httpdsend(id, value, strlen(value), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_mailinterval(int id, const char *name, const char *value)
{
	/* POP3 Interval Time */
	uitoa((unsigned int)get_param_integer(PARAM_MAIL_DELAY), sendbuf);
	cnet_httpdsend(id, sendbuf, strlen(sendbuf), 1);
}
/*----------------------------------------------------------------------------*/
static void
send_libversion(int id, const char *name, const char *value)
{
	const char		*lib_version	= get_libversion();

	/* Library version */
	cnet_httpdsend(id, lib_version, strlen(lib_version), 0);
}
/*----------------------------------------------------------------------------*/
static void
set_myaddr(int id, const char *name, const char *value)
{
	/* IP address */
	if (value != (const char *)0) {
		if (strrchr(value, '.') != (char *)0) {
			set_param_integer(PARAM_MYIP, inet_addr(value));
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_netmask(int id, const char *name, const char *value)
{
	/* Netmask */
	if (value != (const char *)0) {
		if (strrchr(value, '.') != (char *)0) {
			set_param_integer(PARAM_NETMASK, inet_addr(value));
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_gateway(int id, const char *name, const char *value)
{
	/* GW */
	if (value != (const char *)0) {
		if (strrchr(value, '.') != (char *)0) {
			set_param_integer(PARAM_GATEWAY, inet_addr(value));
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_smtpaddr(int id, const char *name, const char *value)
{
	/* SMTP server address */
	if (value != (const char *)0) {
		if (strrchr(value, '.') != (char *)0) {
			set_param_string(PARAM_SMTP_SERVER, value, MAX_SMTPSERVER_LEN);
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_mailfrom(int id, const char *name, const char *value)
{
	/* MAIL FROM */
	if (value != (const char *)0) {
		set_param_string(PARAM_RCV_MAIL_ADDRESS, value, MAX_MAILFROM_LEN);
	}
}
/*----------------------------------------------------------------------------*/
static void
set_mailto(int id, const char *name, const char *value)
{
	/* MAIL TO */
	if (value != (const char *)0) {
		set_param_string(PARAM_FWD_MAIL_ADDRESS, value, MAX_MAILTO_LEN);
	}
}
/*----------------------------------------------------------------------------*/
static void
set_pop3addr(int id, const char *name, const char *value)
{
	/* POP3 server address */
	if (value != (const char *)0) {
		if (strrchr(value, '.') != (char *)0) {
			set_param_string(PARAM_POP3_SERVER, value, MAX_POP3SERVER_LEN);
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_popuser(int id, const char *name, const char *value)
{
	/* POP3 user */
	if (value != (const char *)0) {
		set_param_string(PARAM_POP3_USER, value, MAX_POPUSER_LEN);
	}
}
/*----------------------------------------------------------------------------*/
static void
set_poppasswd1(int id, const char *name, const char *value)
{
	/* POP3 password */
	if (value == (const char *)0) {
		pop3passwd[0] = '\0';
	} else {
		strncpy(pop3passwd, value, MAXPASSWDLEN);
	}
}
/*----------------------------------------------------------------------------*/
static void
set_poppasswd2(int id, const char *name, const char *value)
{
	/* POP3 password(reinput value) */
	if (pop3passwd[0] != '\0' && value != (const char *)0) {
		if (strcmp(pop3passwd, value) == 0) {
			set_param_string(PARAM_POP3_PASSWORD, pop3passwd, MAX_POPPASS_LEN);
		}
	}
}
/*----------------------------------------------------------------------------*/
static void
set_mailinterval(int id, const char *name, const char *value)
{
	/* MAIL interval */
	if (value != (const char *)0) {
		set_param_integer(PARAM_MAIL_DELAY, atoi(value));
	}
}
/*----------------------------------------------------------------------------*/
static void
mail_start(int id, const char *name, const char *value)
{
#if CNET_MAILSAMPLE
	/* start mail process */
	mail_ready();
#endif	/* CNET_MAILSAMPLE */
}
/*----------------------------------------------------------------------------*/
static void
write_param(int id, const char *name, const char *value)
{
	led_on(LED_ALL);
	param_save();
	led_off(LED_ALL);
}
/*----------------------------------------------------------------------------*/
static void
output_serial(int id, const char *name, const char *value)
{
	/* Serial operation */
	SERIAL_INIT(UART_SPEED_9600|UART_PARITY_NONE|UART_CHARLEN_8|UART_STOPBIT_1);
	SERIAL_SEND((UART_CHAR *)"=== SERIAL OPERATION ===\r\n", 26);
	if (value != (const char *)0) {
		SERIAL_SEND((UART_CHAR *)value, strlen(value));
	}
	SERIAL_SEND((UART_CHAR *)"\r\n\r\n", 4);
}
/*----------------------------------------------------------------------------*/
static void
led_control(int id, const char *name, const char *value)
{
	unsigned char	req	= 0;

	/* set request */
	if (!strcmp(name, "led1")) {
		req = LED1;
	}
	else if (!strcmp(name, "led2")) {
		req = LED2;
	}
	else if (!strcmp(name, "led3")) {
		req = LED3;
	}
	else if (!strcmp(name, "led4")) {
		req = LED4;
	}
	else {
		goto end;
	}

	/* LED control */
	if (value != (const char *)0) {
		if (*value == '1') {
			led_on(req);
		}
		else if (*value == '0') {
			led_off(req);
		}
	}

end:
	return;
}
/*----------------------------------------------------------------------------*/
static void
send_dipswst(int id, const char *name, const char *value)
{
	unsigned char	st		= 0;
	unsigned char	mask	= 0;

	/* output switch status */
	if (value == (const char *)0) {
		goto end;
	}
	if (*value == '1') {
		mask = 0x01;
	}
	else if (*value == '2') {
		mask = 0x02;
	}
	else if (*value == '3') {
		mask = 0x04;
	}
	else {
		goto end;
	}

	/* get status */
	st = dipsw_read();

	if (st & mask) {
		cnet_httpdsend(id, "OFF", 3, 0);
	}
	else {
		cnet_httpdsend(id, "ON", 2, 0);
	}

end:
	return;
}
/*----------------------------------------------------------------------------*/
const cnet_finfo			httpd_flist[] = {
	{"val_ifmyaddr",		send_ifmyaddr},
	{"val_ifnetmask",		send_ifnetmask},
	{"val_ifgateway",		send_ifgateway},
	{"val_dhcp",			send_dhcp},
	{"val_myaddr",			send_myaddr},
	{"val_netmask",			send_netmask},
	{"val_gateway",			send_gateway},
	{"val_macaddr",			send_macaddr},
	{"val_smtpserver",		send_smtpaddr},
	{"val_recvmail",		send_mailfrom},
	{"val_fwdmail",			send_mailto},
	{"val_pop3server",		send_pop3addr},
	{"val_pop3user",		send_popuser},
	{"val_pop3interval",	send_mailinterval},
	{"val_libversion",		send_libversion},
	{"in_myaddr",			set_myaddr},
	{"in_netmask",			set_netmask},
	{"in_gateway",			set_gateway},
	{"in_smtpserver",		set_smtpaddr},
	{"in_recvmail",			set_mailfrom},
	{"in_fwdmail",			set_mailto},
	{"in_pop3server",		set_pop3addr},
	{"in_pop3user",			set_popuser},
	{"in_pop3pass1",		set_poppasswd1},
	{"in_pop3pass2",		set_poppasswd2},
	{"in_pop3interval",		set_mailinterval},
	{"mail_start",			mail_start},
	{"write_param",			write_param},
	{"in_serialdata",		output_serial},
	{"led1",				led_control},
	{"led2",				led_control},
	{"led3",				led_control},
	{"led4",				led_control},
	{"dipswst",				send_dipswst},
	{NULL,					NULL}
};
/*----------------------------------------------------------------------------*/
#endif	/* CNET_HTTPD */
