/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : userap.c                                                      */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              User Application Source Code                                  */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "lwip/sys.h"
#include "arch/lib.h"
#include "api/cnet_systimer.h"
#include "api/cnet_pop3c.h"
#include "api/cnet_smtpc.h"
#include "cnet_param.h"
#include "cnet_led.h"

#if CNET_MAILSAMPLE
/*----------------------------------------------------------------------------*/
#define	SAMPLEMAIL_NAME_SUBJECT		"Subject: "
#define	SAMPLEMAIL_SIZE_WORKBUF		128
#define	SAMPLEMAIL_MAX_SUBJECT		80

static unsigned int			smtpaddr								= 0;
static unsigned int			pop3addr								= 0;
static unsigned int			pop3interval							= 0;
static unsigned int			pop3timer								= 0;
static char					wkbuf1[SAMPLEMAIL_SIZE_WORKBUF + 1]		= {0};
static char					subjectbuf[SAMPLEMAIL_MAX_SUBJECT + 1]	= {0};
static unsigned char		mail_sample_state						= 0;
static const unsigned char	led_req[]	= {LED1, LED2, LED3, LED4};
/*----------------------------------------------------------------------------*/
static
int ledcontrol()
{
	const char*		pc		= (const char *)0;
	int				i		= 0;
	unsigned char	on_req	= 0;
	unsigned char	off_req	= 0;

	/* Retrieval of data for LED control */
	for (pc = &wkbuf1[3]; *pc == ' '; pc++);

	/* LED control */
	for (i = 0; i < 4 && *(pc + i) != '\0'; i++) {
		if (*(pc + i) == '1') {
			on_req |= led_req[i];
		}
		else if (*(pc + i) == '0') {
			off_req |= led_req[i];
		}
	}
	led_off(off_req);
	led_on(on_req);

	return (0);
}
/*----------------------------------------------------------------------------*/
static
int smtpforward()
{
	int				retc		= 0;
	int				pop3retc	= 0;

	/* Mail Sending beginning */
	if (cnet_smtpinit(smtpaddr, 25, get_param_string(PARAM_HOSTNAME)) != 0) {
		retc = -1;
		goto end3;
	}
	if ( cnet_smtpdatafirst(
			get_param_string(PARAM_RCV_MAIL_ADDRESS),
			get_param_string(PARAM_FWD_MAIL_ADDRESS), (const char *)0, 0) != 0) {
		retc = -1;
		goto end2;
	}

	/* Transmission of header information */
	if (cnet_smtpdatanext("From: ", 6) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext(get_param_string(PARAM_RCV_MAIL_ADDRESS), (unsigned short)strlen(get_param_string(PARAM_RCV_MAIL_ADDRESS))) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext("\r\n", 2) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext("To: ", 4) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext(get_param_string(PARAM_FWD_MAIL_ADDRESS), (unsigned short)strlen(get_param_string(PARAM_FWD_MAIL_ADDRESS))) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext("\r\n", 2) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext(SAMPLEMAIL_NAME_SUBJECT, (unsigned short)strlen(SAMPLEMAIL_NAME_SUBJECT)) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext(subjectbuf, (unsigned short)strlen(subjectbuf)) != 0) {
		retc = -1;
		goto end1;
	}
	if (cnet_smtpdatanext("\r\n\r\n", 4) != 0) {
		retc = -1;
		goto end1;
	}

	/* Message forwarding */
	do {
		/* Acquisition of reception message */
		pop3retc = cnet_pop3getmsgnext(wkbuf1, SAMPLEMAIL_SIZE_WORKBUF, POP3_MODE_BINARY);
		if (pop3retc < 0) {
			retc = -1;
			goto end1;
		}
		/* Transmission of acquired message to forwarding destination */
		if (cnet_smtpdatanext(wkbuf1, (unsigned short)strlen(wkbuf1)) != 0) {
			retc = -1;
			goto end1;
		}
	} while(pop3retc == 0);

end1:
	/* Mail Sending end */
	if (cnet_smtpdataend() != 0) {
		retc = -1;
	}

end2:
	cnet_smtpend(0);

end3:
	return (retc);
}
/*----------------------------------------------------------------------------*/
void
mail_ready()
{
	mail_sample_state = 1;
}
/*----------------------------------------------------------------------------*/
void
mail_sample()
{
	char			*pc			= (char *)0;
	int				pop3retc	= 0;
	unsigned int	msgno		= 0;
	unsigned int	num			= 0;

	if (mail_sample_state == 0) {
		return;
	}
	if (elapsed_systime(pop3timer) < pop3interval) {
		return;
	}

	pop3retc = 0;

	/* Setting of address and interval information */
	smtpaddr = inet_addr(get_param_string(PARAM_SMTP_SERVER));
	pop3addr = inet_addr(get_param_string(PARAM_POP3_SERVER));
	pop3interval = get_param_integer(PARAM_MAIL_DELAY) * 1000;

	/* Mail reception beginning */
	if (cnet_pop3init(pop3addr, 110) != 0) {
		pop3retc = -1;
		goto end2;
	}

	/* POP attestation */
	if (cnet_pop3auth(get_param_string(PARAM_POP3_USER), get_param_string(PARAM_POP3_PASSWORD)) != 0) {
		pop3retc = -1;
		goto end1;
	}

	/* Confirmation mail's whether it having reached */
	num = 0;
	if (cnet_pop3stat(&num, (unsigned int *)0) != 0) {
		pop3retc = -1;
		goto end1;
	}

	for (msgno = 1; msgno <= num; msgno++) {
		/* Mail acquisition beginning */
		pop3retc = cnet_pop3getmsgfirst(msgno, 0, wkbuf1, SAMPLEMAIL_SIZE_WORKBUF, POP3_MODE_TRUNC);
		if (pop3retc < 0) {
			goto end1;
		}
		/* Acquisition of header information */
		subjectbuf[0] = '\0';
		while(strcmp(wkbuf1, "\r\n") && pop3retc == 0) {
			/* The content is reserved for Subject.  */
			if (!strncmp(wkbuf1, SAMPLEMAIL_NAME_SUBJECT, strlen(SAMPLEMAIL_NAME_SUBJECT))) {
				strncpy(subjectbuf, &wkbuf1[strlen(SAMPLEMAIL_NAME_SUBJECT)], SAMPLEMAIL_MAX_SUBJECT);
				subjectbuf[SAMPLEMAIL_MAX_SUBJECT] = '\0';
				if ((pc = (char*)strstr(subjectbuf, "\r\n")) != (char *)0) {
					*pc = '\0';
				}
			}
			/* Acquisition of the following line */
			pop3retc = cnet_pop3getmsgnext(wkbuf1, SAMPLEMAIL_SIZE_WORKBUF, POP3_MODE_TRUNC);
			if (pop3retc < 0) {
				goto end1;
			}
		}
		if (pop3retc == 0) {
			/* Acquisition of the first line of message */
			pop3retc = cnet_pop3getmsgnext(wkbuf1, SAMPLEMAIL_SIZE_WORKBUF, POP3_MODE_TRUNC);
			if (pop3retc < 0) {
				goto end1;
			}
			if (!strncmp(wkbuf1, "LED", 3)) {
				/* LED control */
				if (ledcontrol() != 0) {
					pop3retc = -1;
					goto end1;
				}
			}
			else if (!strncmp(wkbuf1, "FWD", 3)) {
				if (pop3retc == 0) {
					/* Mail forwarding */
					if (smtpforward() != 0) {
						pop3retc = -1;
						goto end1;
					}
				}
			}
		}

		/* Mail acquisition end */
		if (cnet_pop3getmsgend() != 0) {
			pop3retc = -1;
			goto end1;
		}

		/* Deletion of mail */
		if (cnet_pop3delete(msgno) != 0) {
			pop3retc = -1;
			goto end1;
		}
	}

end1:
	/* Mail reception end */
	cnet_pop3end(0);

end2:
	/* When making an error, LED1-4 is blinked three times. */
	if (pop3retc < 0) {
		for (num = 3; num > 0; num--) {
			led_on(LED_ALL);
			msleep(800);
			led_off(LED_ALL);
			msleep(800);
		}
	}

	pop3timer = get_systimer();
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_MAILSAMPLE */
