/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : config.c                                                      */
/* Type       : C Programming Language Source Code                            */
/* Version    : E1.00                                                         */
/* Date       : 2006 SEP 15                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Configuration Source Code                                     */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "lwip/sys.h"
#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/pbuf.h"
#include "lwip/udp.h"
#include "lwip/tcp.h"
#include "lwip/api.h"
#include "lwip/api_msg.h"
#include "lwip/tcpip.h"
#if MEMP_NUM_NETCONN > 0
#include "lwip/sockets.h"
#endif
#include "lwip/raw.h"
#include "netif/etharp.h"
#include "arch/cnet_functions.h"

#define MEM_ALIGN_SIZE(size) (((size) + MEM_ALIGNMENT - 1) & ~(MEM_ALIGNMENT - 1))

const u16_t	lwip_opt[] = {
	MEM_ALIGNMENT,
	MEM_SIZE,
	MEMP_NUM_NETCONN,
	PBUF_POOL_SIZE,
	PBUF_POOL_BUFSIZE,
	PBUF_LINK_HLEN,
	TCP_TTL,
	TCP_MSS,
	TCP_SND_BUF,
	TCP_SND_QUEUELEN,
	TCP_WND,
	TCP_MAXRTX,
	TCP_SYNMAXRTX,
	TCP_SNDLOWAT,
	ARP_TABLE_SIZE,
	ICMP_TTL,
	UDP_TTL,
	RAW_TTL
};

const u16_t	memp_num[MEMP_MAX] = {
	MEMP_NUM_PBUF,
	MEMP_NUM_RAW_PCB,
	MEMP_NUM_UDP_PCB,
	MEMP_NUM_TCP_PCB,
	MEMP_NUM_TCP_PCB_LISTEN,
	MEMP_NUM_TCP_SEG,
	MEMP_NUM_NETBUF,
	MEMP_NUM_NETCONN,
	MEMP_NUM_API_MSG,
	MEMP_NUM_TCPIP_MSG,
	MEMP_NUM_SYS_TIMEOUT
};

#if (MEMP_NUM_NETCONN > 0)
struct lwip_socket sockets[MEMP_NUM_NETCONN];
#endif	/* MEMP_NUM_NETCONN */

#if (ARP_TABLE_SIZE > 0)
struct etharp_entry arp_table[ARP_TABLE_SIZE];
#endif	/* ARP_TABLE_SIZE */

#if (MALLOC_SIZE > 0)
long				malloc_heap[(MALLOC_SIZE + 3) / 4];
const unsigned int	malloc_heap_size	= (MALLOC_SIZE + 3) / 4;
#endif	/* MALLOC_SIZE */

u8_t	ram[MEM_ALIGN_SIZE(MEM_SIZE + sizeof(struct mem))];

u8_t memp_memory[(MEMP_NUM_PBUF *
			MEM_ALIGN_SIZE(sizeof(struct pbuf) + sizeof(struct memp)) +
		  MEMP_NUM_RAW_PCB *
			MEM_ALIGN_SIZE(sizeof(struct raw_pcb) + sizeof(struct memp)) +
		  MEMP_NUM_UDP_PCB *
			MEM_ALIGN_SIZE(sizeof(struct udp_pcb) + sizeof(struct memp)) +
		  MEMP_NUM_TCP_PCB *
			MEM_ALIGN_SIZE(sizeof(struct tcp_pcb) + sizeof(struct memp)) +
		  MEMP_NUM_TCP_PCB_LISTEN *
			 MEM_ALIGN_SIZE(sizeof(struct tcp_pcb_listen) + sizeof(struct memp)) +
		  MEMP_NUM_TCP_SEG *
			 MEM_ALIGN_SIZE(sizeof(struct tcp_seg) + sizeof(struct memp)) +
		  MEMP_NUM_NETBUF *
			 MEM_ALIGN_SIZE(sizeof(struct netbuf) + sizeof(struct memp)) +
		  MEMP_NUM_NETCONN *
			 MEM_ALIGN_SIZE(sizeof(struct netconn) + sizeof(struct memp)) +
		  MEMP_NUM_API_MSG *
			 MEM_ALIGN_SIZE(sizeof(struct api_msg) + sizeof(struct memp)) +
		  MEMP_NUM_TCPIP_MSG *
			 MEM_ALIGN_SIZE(sizeof(struct tcpip_msg) + sizeof(struct memp)) +
		  MEMP_NUM_SYS_TIMEOUT *
			 MEM_ALIGN_SIZE(sizeof(struct sys_timeout) + sizeof(struct memp)))];

u8_t	pbuf_pool_memory[(PBUF_POOL_SIZE * MEM_ALIGN_SIZE(PBUF_POOL_BUFSIZE + sizeof(struct pbuf))) + MEM_ALIGNMENT];

//#if IP_REASSEMBLY           /* NEC */
//u8_t	buf[MEM_ALIGN_SIZE(MAX_MTU)];	/* for ip_frag.c */
//#endif

#if	CONFIG_RAW_IMPLEMENT
extern const struct raw_functions	raw_functions;
#endif	/* CONFIG_RAW_IMPLEMENT */

#if	CONFIG_ETHARP_IMPLEMENT
extern const struct etharp_functions	etharp_functions;
#endif	/* CONFIG_ETHARP_IMPLEMENT */

const struct cnet_functions	cnet_functions	= {
#if	CONFIG_RAW_IMPLEMENT
	&raw_functions,
#else	/* CONFIG_RAW_IMPLEMENT */
	(struct raw_functions *)0,
#endif	/* CONFIG_RAW_IMPLEMENT */
#if	CONFIG_ETHARP_IMPLEMENT
	&etharp_functions
#else	/* CONFIG_ETHARP_IMPLEMENT */
	(struct etharp_functions *)0
#endif	/* CONFIG_ETHARP_IMPLEMENT */
};
/*----------------------------------------------------------------------------*/
u16_t
mem_align_size(u16_t size)
{
	return MEM_ALIGN_SIZE(size);
}
/*----------------------------------------------------------------------------*/
