#   Copyright (C) NEC Electronics Corporation 1998,2002
#   NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY
#   All rights reserved by NEC Electronics Corporation.
#   This program must be used solely for the purpose for which
#   it was furnished by NEC Electronics Corporation.  No part of this
#   program may be reproduced or disclosed to others, in any
#   form, without the prior written permission of NEC Electronics
#   Corporation.  Use of copyright notice does not evidence
#   publication of the program.

#	@(#)crtE.s	1.8 02/12/12 15:19:37

#=============================================================================
# NAME
#    crtE.s -  start up module for ca850(V850E)
#
# DESCRIPTIONS:
#      This assembly program is a sample of start-up module for ca850(V850E).
#    If you modified this program, you must assemble this file, and
#    locate a given directory.
#
#    Unless -G is specified, sections are located as the following.
#
#                  |        :        |
#                  |        :        |
#	    tp -> -+-----------------+ __start     __tp_TEXT
#                  |  start up       |
#                  |---------------- |
#   text section   |                 |
#                  | user program    |
#                  |                 |
#                  |-----------------|
#                  | library         |
#	          -+-----------------+ 
#                  |        :        |
#                  |        :        |
#	          -+-----------------+ __argc
#                  |      0          |
#                  |---------------- | __argv
#   data section   |    #.L16        |
#                  |---------------- | .L16
#                  | 0x0,0x0,0x0,0x0 |
#                 -+-----------------+               
#                  |                 |
#   sdata section  |                 |
#                  |                 |
#            gp-> -+-----------------+             __ssbss
#                  |                 |
#   sbss section   |                 |
#                  |                 |
#                  +-----------------+ __stack     __esbss     __sbss
#                  | stack area      |
#   bss section    |                 |
#                  |  0x200 bytes    |
#            sp-> -+-----------------+ __stack + STACKSIZE     __ebss
#
#=============================================================================


#-----------------------------------------------------------------------------
#	special symbols
#-----------------------------------------------------------------------------
	.extern __tp_TEXT, 4
	.extern __gp_DATA, 4
	.extern __ep_DATA, 4
	.extern	__ssbss, 4
	.extern __esbss, 4
	.extern __sbss, 4
	.extern __ebss, 4

	
#-----------------------------------------------------------------------------
#	 C program main function
#-----------------------------------------------------------------------------
	.extern	_main


#-----------------------------------------------------------------------------
#	for argv
#-----------------------------------------------------------------------------
	.data
	.size	__argc, 4
	.align	4
__argc:
	.word	0
	.size	__argv, 4
__argv:
	.word	#.L16
.L16:
	.byte	0
	.byte	0
	.byte	0
	.byte	0


#-----------------------------------------------------------------------------
#	dummy data declaration for creating sbss section
#-----------------------------------------------------------------------------
	.sbss
	.lcomm	__sbss_dummy, 0, 0

	
#-----------------------------------------------------------------------------
#	system stack
#-----------------------------------------------------------------------------
	.set	STACKSIZE, 0x800
	.bss
	.lcomm	__stack, STACKSIZE, 4

#-----------------------------------------------------------------------------
#	system heap
#-----------------------------------------------------------------------------
	.set	HEAPSIZE, 0x200
	.globl	___sysheap
	.bss
	.lcomm	___sysheap, HEAPSIZE, 4
	.data
	.globl	___sizeof_sysheap
___sizeof_sysheap:
	.word	HEAPSIZE

#-----------------------------------------------------------------------------
#	SECURITY_ID
#-----------------------------------------------------------------------------
#	.section	"SECURITY_ID"
#	.word		0xFFFFFFFF
#	.word		0xFFFFFFFF
#	.hword		0xFFFF

#-----------------------------------------------------------------------------
#	RESET handler
#-----------------------------------------------------------------------------

	.section	"RESET", text
	jr			__start
	
#-----------------------------------------------------------------------------
#	start up
#		pointers:  tp - text pointer
#			   gp - global pointer
#			   sp - stack pointer
#			   ep - element pointer
#		mask reg:  r20 - 0xff
#			   r21 - 0xffff
#	exit status is set to r10
#-----------------------------------------------------------------------------
	.text
	.align	4
	.globl	__start
	.globl  __exit
	.globl	__startend
#	.extern  ___PROLOG_TABLE
__start:
	mov	#__tp_TEXT, tp		-- set tp register
	mov	#__gp_DATA, gp		-- set gp register offset
	add	tp, gp			-- set gp register
	mov	#__stack+STACKSIZE, sp	-- set sp register
	mov	#__ep_DATA, ep		-- set ep register
#
	.option nowarning
	mov	0xff,   r20		-- set mask register
	mov	0xffff, r21		-- set mask register
	.option warning
#
	mov	#__ssbss, r13		-- clear sbss section
	mov	#__esbss, r12
	cmp	r12, r13
	jnl	.L11
.L12:
	st.w	r0, [r13]
	add	4, r13
	cmp	r12, r13
	jl	.L12
.L11:
#
	mov	#__sbss, r13		-- clear bss section
	mov	#__ebss, r12
	cmp	r12, r13
	jnl	.L14
.L15:
	st.w	r0, [r13]
	add	4, r13
	cmp	r12, r13
	jl	.L15
.L14:
#
#	mov     #___PROLOG_TABLE, r12   -- for prologue/epilogue runtime
#	ldsr    r12, 20			-- set CTBP (CALLT base pointer)
#
#					-- for Programmable peripheral I/O 
#	mov	0x9234, r13		-- 0x1234(addr) | 0x8000(use pI/Or)
#	st.h	r13, BPC		-- set BPC
#
	jarl	___Init_tk850_jg2, lp
#
	.extern __S_romp, 4
	mov		#__S_romp, r6
	mov		-1, r7
	jarl	__rcopy, lp
#
	ld.w	$__argc, r6		-- set argc
	movea	$__argv, gp, r7		-- set argv
	jarl	_main, lp		-- call main function
__exit:
	halt				-- end of program
__startend:

#-----------------------------------------------------------------------------
#	initialize TK-850/JG2+NET
#-----------------------------------------------------------------------------
___Init_tk850_jg2:
	mov		0x01, r11
	st.b	r11, VSWC

	mov		0x00, r11
	st.b	r11, PRCMD
	st.b	r11, OCDM
	nop
	nop
	nop
	nop
	nop

	mov		0x00, r11
	st.b	r11, WDTM2

	mov		0x00, r11
	st.b	r11, PRCMD
	st.b	r11, PCC
	nop
	nop
	nop
	nop
	nop

	mov		0x55, r11
__wait_ost:
	nop
	nop
	nop
	sub		1, r11
	jnz		__wait_ost

__wait_clock:
	tst1	0, LOCKR
	jnz		__wait_clock

	mov		0x03, r11
	st.b	r11, PLLCTL

	mov		0x02, r11
	st.b	r11, PMCCM

	mov		0x0000, r11
	st.h	r11, PF9

	mov		0x2000, r11
	st.h	r11, PFC9

	mov		0x0003, r11
	st.h	r11, PFCE9

	mov		0x21ff, r11
	st.h	r11, PMC9

	mov		0xfdff, r11
	st.h	r11, PM9

	mov		0xffff, r11
	st.h	r11, PMCDL

	mov		0x11, r11
	st.b	r11, PMCCT

	mov		0xbf, r11
	st.b	r11, PMCT

	mov		0x01, r11
	st.b	r11, EXIMC
	nop

	mov		0x5555, r11
	st.h	r11, BSC

	mov		0x0000, r11
	st.h	r11, DWC0

	mov		0x10, r11
	st.b	r11, INTF0

	mov		0x10, r11
	st.b	r11, INTR0

	mov		0x00, r11
	st.b	r11, INTF9H

	mov		0x20, r11
	st.b	r11, INTR9H

	mov		0x00, r11
	st.b	r11, PMCDH

	mov		0xC0, r11
	st.b	r11, PMDH

	mov		0x1F, r11
	st.b	r11, PDH

	mov		0xFE, r11
	st.b	r11, PM7H

	mov		0x01, r11
	st.b	r11, P7H

	mov		0xFF, r11
	st.b	r11, PM7L

	mov		0x00, r11
	st.b	r11, PFC4

	mov		0x07, r11
	st.b	r11, PMC4

	mov		0x10, r11
	st.b	r11, PMC0

	mov		0x03, r11
	st.b	r11, PFC5

	mov		0x00, r11
	st.b	r11, PFCE5

	mov		0x03, r11
	st.b	r11, PMC5

	mov		0x03, r11
	st.b	r11, KRM

	mov		0x1387, r11
	st.h	r11, TQ0CCR0

	mov		0xffff, r11
	st.h	r11, TQ0CCR1

	mov		0xffff, r11
	st.h	r11, TQ0CCR2

	mov		0xffff, r11
	st.h	r11, TQ0CCR3

	mov		0x82, r11
	st.b	r11, TQ0CTL0

	mov		0x00, r11
	st.b	r11, TQ0CCIC0

	jmp	[lp]

#								#
#-------------------- end of start up module -------------------#
#								#
