/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : uartsetup.c                                                   */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Uart Setup Source Code                                        */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "lwip/netif.h"
#include "lwip/inet.h"
#include "arch/lib.h"
#include "cnet_uart.h"
#include "cnet_param.h"
#include "cnet_led.h"

#if CNET_UARTSETUP
/*----------------------------------------------------------------------------*/
#define	SETUP_UART_INIT			UART_INIT(2)
#define	SETUP_UART_SEND			UART_SEND(2)
#define	SETUP_UART_RECVSTART	UART_RECVSTART(2)
#define	SETUP_UART_RECVSTOP		UART_RECVSTOP(2)
#define	SETUP_UART_RECV_TIMEOUT	UART_RECV_TIMEOUT(2)
#define	SETUP_UART_OPTION		UART_OPTION(2)
#define	SETUPBUFLEN				256
typedef	unsigned char			UART_CHAR;

static char		*setupbuf	= (char *)0;

/*----------------------------------------------------------------------------*/
void
uart_printinfo(struct netif *netif)
{
	int				len	= 0;
	char			*p	= (char *)0;
	struct in_addr	addr;

	if (setupbuf == (char *)0) {
		return;
	}

	SETUP_UART_INIT(UART_SPEED_9600|UART_PARITY_NONE|UART_CHARLEN_8|UART_STOPBIT_1);
	SETUP_UART_SEND((UART_CHAR *)"\r\n======== INFORMATION START ========\r\n", 39);

	SETUP_UART_SEND((UART_CHAR *)"DHCP Function   : ", 18);
	SETUP_UART_SEND((UART_CHAR *)(get_param_integer(PARAM_DHCP) ? "ON \r\n" : "OFF\r\n"), 5);

	SETUP_UART_SEND((UART_CHAR *)"My IP Address   : ", 18);
	addr.s_addr = netif->ip_addr.addr;
	p = inet_ntoa(addr);
	SETUP_UART_SEND((UART_CHAR *)p, strlen(p));
	SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);

	SETUP_UART_SEND((UART_CHAR *)"Netmask         : ", 18);
	addr.s_addr = netif->netmask.addr;
	p = inet_ntoa(addr);
	SETUP_UART_SEND((UART_CHAR *)p, strlen(p));
	SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);

	SETUP_UART_SEND((UART_CHAR *)"GW IP Address   : ", 18);
	addr.s_addr = netif->gw.addr;
	p = inet_ntoa(addr);
	SETUP_UART_SEND((UART_CHAR *)p, strlen(p));
	SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);

	SETUP_UART_SEND((UART_CHAR *)"MAC Address     : ", 18);
	/* MAC Address will be changed into text form. */
	len = conv_macaddr(setupbuf, netif->hwaddr);
	SETUP_UART_SEND((UART_CHAR *)setupbuf, len);
    SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);

	SETUP_UART_SEND((UART_CHAR *)"Library version : ", 18);
	p = (char *)get_libversion();
	SETUP_UART_SEND((UART_CHAR *)p, strlen(p));
    SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);

	SETUP_UART_SEND((UART_CHAR *)"======== INFORMATION END   ========\r\n\r\n", 39);

	free(setupbuf);
	setupbuf = (char *)0;
}
/*----------------------------------------------------------------------------*/
void
uart_setupmon(void)
{
	unsigned short	len;

	setupbuf = malloc(SETUPBUFLEN);
	if (setupbuf == (char *)0) {
		return;
	}

	SETUP_UART_INIT(UART_SPEED_9600|UART_PARITY_NONE|UART_CHARLEN_8|UART_STOPBIT_1);
	SETUP_UART_OPTION(ECHO_BACK_MODE|UART_FC_NONE);
	SETUP_UART_SEND((UART_CHAR *)"\r\n\r\n--- SETUP MONITOR ---", 25);

	while (1) {
		/* Send menu */
		if (get_param_integer(PARAM_DHCP) == 0) {
			SETUP_UART_SEND((UART_CHAR *)"\r\n\r\n1) DHCP Function Set ON/OFF [OFF]\r\n", 39);
		} else {
			SETUP_UART_SEND((UART_CHAR *)"\r\n\r\n1) DHCP Function Set ON/OFF [ON]\r\n", 38);
		}
		SETUP_UART_SEND((UART_CHAR *)"2) My IP Address Change\r\n", 25);
		SETUP_UART_SEND((UART_CHAR *)"3) Netmask Change\r\n", 19);
		SETUP_UART_SEND((UART_CHAR *)"4) GW IP Address Change\r\n", 25);
		SETUP_UART_SEND((UART_CHAR *)"5) Saves to the FlashROM\r\n", 26);
		SETUP_UART_SEND((UART_CHAR *)"other) Boot\r\n\r\n", 15);
		SETUP_UART_SEND((UART_CHAR *)"Select number (1-5, other): ", 28);

		/* Receive number */
		SETUP_UART_RECVSTART();
		len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 2, 20000);
		SETUP_UART_RECVSTOP();
		SETUP_UART_SEND((UART_CHAR *)"\r\n\r\n", 4);
		if (len <= 0) {
			SETUP_UART_SEND((UART_CHAR *)"Timeout. ", 9);
			break;
		}
		if (len != 2 || setupbuf[1] != '\r') {
			break;
		}

		switch ((int)*setupbuf) {
		case '1':
			/* DHCP Function Set */
			SETUP_UART_SEND((UART_CHAR *)"Choose a DHCP Function (ON:y,OFF:n): ", 37);
			SETUP_UART_RECVSTART();
			len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 2, 20000);
			SETUP_UART_RECVSTOP();
			if (len <= 1) {
				break;
			}
			SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);
			if (*setupbuf == 'y' || *setupbuf == 'Y') {
				set_param_integer(PARAM_DHCP, 1);
			} else if (*setupbuf == 'n' || *setupbuf == 'N') {
				set_param_integer(PARAM_DHCP, 0);
			} else {
				SETUP_UART_SEND((UART_CHAR *)"\r\nFormat Error!\r\n\r\n", 19);
			}
			break;
		case '2':
			/* My IP Address Change */
			SETUP_UART_SEND((UART_CHAR *)"Enter My IP Address (???.???.???.???): ", 39);
			SETUP_UART_RECVSTART();
			len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 16, 20000);
			SETUP_UART_RECVSTOP();
			if (len <= 1) {
				break;
			}
			len--;
			setupbuf[len] = '\0';
			if ((char *)strrchr(setupbuf, '.') == (char *)0) {
				SETUP_UART_SEND((UART_CHAR *)"\r\nFormat Error!\r\n\r\n", 19);
			} else {
				set_param_integer(PARAM_MYIP, inet_addr(setupbuf));
			}
			break;
		case '3':
			/* Netmask Change */
			SETUP_UART_SEND((UART_CHAR *)"Enter Netmask Address (???.???.???.???): ", 41);
			SETUP_UART_RECVSTART();
			len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 16, 20000);
			SETUP_UART_RECVSTOP();
			if (len <= 1) {
				break;
			}
			len--;
			setupbuf[len] = '\0';
			if ((char *)strrchr(setupbuf, '.') == (char *)0) {
				SETUP_UART_SEND((UART_CHAR *)"\r\nFormat Error!\r\n\r\n", 19);
			} else {
				set_param_integer(PARAM_NETMASK, inet_addr(setupbuf));
			}
			break;
		case '4':
			/* GW IP Address Change */
			SETUP_UART_SEND((UART_CHAR *)"Enter GW IP Address (???.???.???.???): ", 39);
			SETUP_UART_RECVSTART();
			len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 16, 20000);
			SETUP_UART_RECVSTOP();
			if (len <= 1) {
				break;
			}
			len--;
			setupbuf[len] = '\0';
			if ((char *)strrchr(setupbuf, '.') == (char *)0) {
				SETUP_UART_SEND((UART_CHAR *)"\r\nFormat Error!\r\n\r\n", 19);
			} else {
				set_param_integer(PARAM_GATEWAY, inet_addr(setupbuf));
			}
			break;
		case '5':
			/* Saves to the FlashROM */
			SETUP_UART_SEND((UART_CHAR *)"Save O.K?(y/n): ", 16);
			SETUP_UART_RECVSTART();
			len = SETUP_UART_RECV_TIMEOUT((UART_CHAR *)setupbuf, 2, 20000);
			SETUP_UART_RECVSTOP();
			if (len <= 1) {
				break;
			}
			SETUP_UART_SEND((UART_CHAR *)"\r\n", 2);
			if (*setupbuf == 'y' || *setupbuf == 'Y') {
				SETUP_UART_SEND((UART_CHAR *)"\r\n\r\nWriting...\r\n\r\n", 18);
				led_on(LED_ALL);
				if (param_save() != 0) {
					SETUP_UART_SEND((UART_CHAR *)"\r\nFlashROM write error!\r\n\r\n", 27);
				}
				led_off(LED_ALL);
			} else if (*setupbuf == 'n' || *setupbuf == 'N') {
			} else {
				SETUP_UART_SEND((UART_CHAR *)"\r\nFormat Error!\r\n\r\n", 19);
			}
			break;
		default:
			goto end;
		}
	}

end:
	SETUP_UART_SEND((UART_CHAR *)"Booting...\r\n\r\n", 14);
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_UARTSETUP */
