/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : cnet_uart.h                                                   */
/* Type       : C Programming Language Header                                 */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Serial API Header                                             */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#ifndef __CNET_UART_H__
#define __CNET_UART_H__

#define	UART_SPEED_300			0x00	/* UAnCTL2 = 0x82, UAnCTL1 = 8 */
#define	UART_SPEED_600			0x01	/* UAnCTL2 = 0x82, UAnCTL1 = 7 */
#define	UART_SPEED_1200			0x02	/* UAnCTL2 = 0x82, UAnCTL1 = 6 */
#define	UART_SPEED_2400			0x03	/* UAnCTL2 = 0x82, UAnCTL1 = 5 */
#define	UART_SPEED_4800			0x04	/* UAnCTL2 = 0x82, UAnCTL1 = 4 */
#define	UART_SPEED_9600			0x05	/* UAnCTL2 = 0x82, UAnCTL1 = 3 */
#define	UART_SPEED_19200		0x06	/* UAnCTL2 = 0x82, UAnCTL1 = 2 */
#define	UART_SPEED_38400		0x07	/* UAnCTL2 = 0x82, UAnCTL1 = 1 */
#define	UART_SPEED_76800		0x08	/* UAnCTL2 = 0x82, UAnCTL1 = 0 */
#define	UART_SPEED_31250		0x09	/* UAnCTL2 = 0xA0, UAnCTL1 = 1 */
#define	UART_SPEED_115200		0x0A	/* UAnCTL2 = 0x57, UAnCTL1 = 0 */
#define	UART_SPEED_153600		0x0B	/* UAnCTL2 = 0x41, UAnCTL1 = 0 */
#define	UART_SPEED_312500		0x0C	/* UAnCTL2 = 0x20, UAnCTL1 = 0 */

#define	UART_PARITY_NONE		0x00	/* UAnPS1 = 0, UAnPS0 = 0 */
#define	UART_PARITY_ZERO		0x40	/* UAnPS1 = 0, UAnPS0 = 1 */
#define	UART_PARITY_ODD			0x80	/* UAnPS1 = 1, UAnPS0 = 0 */
#define	UART_PARITY_EVEN		0xC0	/* UAnPS1 = 1, UAnPS0 = 1 */

#define	UART_CHARLEN_7			0x00	/* UAnCL = 0 */
#define	UART_CHARLEN_8			0x20	/* UAnCL = 1 */

#define	UART_STOPBIT_1			0x00	/* UAnSL = 0 */
#define	UART_STOPBIT_2			0x10	/* UAnSL = 1 */

#define	UART_FC_NONE			0x00	/* none flow control */
#define	UART_FC_HARDWARE		0x10	/* hardware flow control */

/* Send/Receive mode */
#define ECHO_BACK_MODE			0x02	/* echo back Line mode */
#define LINE_MODE				0x01	/* Line mode */
#define BINARY_MODE 			0x00	/* Binary mode */

#define	UART_INIT(p)			uart##p##_init
#define	UART_END(p)				uart##p##_end
#define	UART_SEND(p)			uart##p##_send
#define	UART_RECVSTART(p)		uart##p##_recvstart
#define	UART_RECVSTOP(p)		uart##p##_recvstop
#define	UART_RECV(p)			uart##p##_recv
#define	UART_RECV_TIMEOUT(p)	uart##p##_recv_timeout
#define	UART_OPTION(p)			uart##p##_option

void uart1_init(unsigned char para);
void uart1_end(void);
void uart1_send(const unsigned char *msg, unsigned short len);
void uart1_recvstart(void);
void uart1_recvstop(void);
unsigned short uart1_recv(unsigned char *msg, unsigned short len);
unsigned short uart1_recv_timeout(unsigned char *msg, unsigned short len, unsigned long timeout);
void uart1_option(unsigned char mode);
void uart1_recv_interrupt(void);
void uart1_send_interrupt(void);

void uart2_init(unsigned char para);
void uart2_end(void);
void uart2_send(const unsigned char *msg, unsigned short len);
void uart2_recvstart(void);
void uart2_recvstop(void);
unsigned short uart2_recv(unsigned char *msg, unsigned short len);
unsigned short uart2_recv_timeout(unsigned char *msg, unsigned short len, unsigned long timeout);
void uart2_option(unsigned char mode);
void uart2_recv_interrupt(void);
void uart2_send_interrupt(void);

#endif	/* __CNET_UART_H__ */
