/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : main.c                                                        */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              Main Routine Source Code for OS less                          */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#pragma ioreg

#include "config.h"

#include "lwip/netif.h"
#include "lwip/ip.h"
#include "lwip/ip_addr.h"
#include "api/cnet_init.h"
#include "api/cnet_systimer.h"
#include "api/cnet_single.h"
#include "api/cnet_dhcpc.h"
#include "api/cnet_httpd.h"
#include "cnet_param.h"
#include "cnet_lanc.h"
#include "cnet_uart.h"


/* gx[X{[hWX^`wb_ */
#include "driver/cqbb.h"
static void CPU_Initialize(void);
err_t NICif_Init(struct netif *);
void NIC_Interrupt(void);


/*----------------------------------------------------------------------------*/
#pragma interrupt	INTTQ0CC0	ms_counter
__interrupt void
ms_counter(void)
{
	inc_systimer();
}
/*----------------------------------------------------------------------------*/
//#if CNET_LANC
//#pragma interrupt	INTP4		lanc_interrupt
#pragma interrupt	INTP0		lanc_interrupt
__interrupt void
lanc_interrupt(void)
{
//	lan9115_interrupt();

	unsigned short int stat;
	stat=*CQBB_IntStat;		/* gx[X{[h荞݃Xe[^X擾 */
	if (stat&CQBB_LANInt) {	/* LAN荞ݔI */
		NIC_Interrupt();	/* 荞ݏ֐R[ */
	}
}
//#endif	/* CNET_LANC */
/*----------------------------------------------------------------------------*/
#if CNET_UART1
#pragma interrupt	INTUA1R		SerialRecvIntr1
__interrupt void
SerialRecvIntr1(void)
{
	uart1_recv_interrupt();
}
/*----------------------------------------------------------------------------*/
#pragma interrupt	INTUA1T		SerialSendIntr1
__interrupt void
SerialSendIntr1(void)
{
	uart1_send_interrupt();
}
#endif	/* CNET_UART1 */
/*----------------------------------------------------------------------------*/
#if CNET_UART2
#pragma interrupt	INTUA2R		SerialRecvIntr2
__interrupt void
SerialRecvIntr2(void)
{
	uart2_recv_interrupt();
}
/*----------------------------------------------------------------------------*/
#pragma interrupt	INTUA2T		SerialSendIntr2
__interrupt void
SerialSendIntr2(void)
{
	uart2_send_interrupt();
}
#endif	/* CNET_UART2 */
/*----------------------------------------------------------------------------*/
static void
netif_setup(void)
{
	static struct netif	netif	= {0};
	struct ip_addr		ipaddr;
	struct ip_addr		netmask;
	struct ip_addr		gw;

	/* Setup monitor by UART I/O */
//	uart_setupmon();

	if (get_param_integer(PARAM_DHCP)) {
		/* Use DHCP */
		IP4_ADDR(&gw, 0,0,0,0);
		IP4_ADDR(&ipaddr, 0,0,0,0);
		IP4_ADDR(&netmask, 0,0,0,0);

//		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, lan9115if_init, ip_input);
		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, NICif_Init, ip_input);
		netif_set_default(&netif);

		if (cnet_dhcpstart(&netif) != 0) {
			/* When it doesn't go well, the use of DHCP is stopped. */
			cnet_dhcpstop(&netif);
			gw.addr = get_param_integer(PARAM_GATEWAY);
			ipaddr.addr = get_param_integer(PARAM_MYIP);
			netmask.addr = get_param_integer(PARAM_NETMASK);
			netif_set_addr(&netif, &ipaddr, &netmask, &gw);
			netif_set_up(&netif);
		}
	}
	else {
		/* Not use DHCP */
		gw.addr = get_param_integer(PARAM_GATEWAY);
		ipaddr.addr = get_param_integer(PARAM_MYIP);
		netmask.addr = get_param_integer(PARAM_NETMASK);

//		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, lan9115if_init, ip_input);
		netif_add(&netif, &ipaddr, &netmask, &gw, NULL, NICif_Init, ip_input);
		netif_set_default(&netif);
		netif_set_up(&netif);
	}

	/* UART output of setup result */
//	uart_printinfo(&netif);
}

/*----------------------------------------------------------------------------*/
int
main(int argc, char **argv)
{
	/* Load the Network Parameter of FlashROM to RAM. */
	param_load();

	__DI();
	
	CPU_Initialize();

	PMCT.6 = 0;
	
	__EI();

	/* Network initialization */
	cnet_init();

	/* Setting of network interface */
	netif_setup();

	/* Start of HTTP server */
	cnet_httpdinit();

	while (1) {
		/* Execution of sample program of mail processing */
//		mail_sample();
		cnet_single_routine();
		msleep(1);
	}

	return 0;
}

/*----------------------------------------------------------------------------*/
void CPU_Initialize(void)
{
	unsigned short *p;

	/* NbN4{[h */
	PRCMD = 0;
	PCC = 0;
	while(LOCKR);
	PLLCTL |= (1<<1);

	/* OoXRg[ */
	EXIMC=1;		/* Zp[goXI */
	__asm("nop");	/* NOP */
	BSC  =0x5545;	/* ubN2F8rbg  ȊO:16rbg */
	DWC0 =0x7220;	/* oXANZXEFCgݒ(ubN0: ȊO:ᑬ) */
	AWC  =0xFF00;	/* AhXEFCg}Ȃ */
	BCC  =0xAAA8;	/* AChXe[g}(ubN0:Ȃ ȊO:) */

	/* |[gݒ	*/
	PFCE9=0x0000;	/* AhXoXA15`A0o  */
	PFC9 =0x0000;	/* AhXoXA15`A0o  */
	PMC9 =0xFFFF;	/* AhXoXA15`A0o  */
	PMCDH=0x3F;		/* AhXoXA21`A16o */
	PMCDL=0xFFFF;	/* f[^oXD15`D0I */
	PMCCM=3;		/* CLKo&WAIT */
	PMCCT=0x13;		/* /RD&/WR1&/WR0o */

	/* O荞ݓ͏ */
	PMC0 =0x78;		/* INTP0`3 */
	PFC0 =0;		/* INTP0 */
	INTR0=0x00;
	INTF0=0x78;		/* INTP0`3 GbWo */
	NFC  =0;		/* AiOmCY */

	/* CPUO荞݃Rg[ */
	PIC0 =0x01;	/* 荞݋/x1 */
	PIC1 =0x47;	/* 荞݃}XN/xŒ */
	PIC2 =0x47;	/* 荞݃}XN/xŒ */
	PIC3 =0x47;	/* 荞݃}XN/xŒ */

	/* gx[X{[hLANRg[荞ݏ */
	*CQBB_LANIntCtl	= LAN_IntEna | LAN_IntSta;	/* 荞݃Cl[u&荞݃NA */

	TQ0CTL0		= 0x02;								/* 5MHz						*/
	TQ0CCR0		= 4999;								/* 200ns * 4999				*/
	TQ0CCR1		= 4999;								/* 200ns * 4999				*/
	TQ0CTL0		= 0x82;								/* Enable					*/
	TQ0CCIF0	= 0;
	TQ0CCIF1	= 0;
	TQ0CCMK0	= 0;
}
