/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : m1xta01a_tk850_jg2.c                                          */
/* Type       : C Programming Language Source Code                            */
/* Version    : E1.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              M1XTA01A Source Code for TK-850/JG2+NET                       */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "cnet_uart.h"
#include "netif/ppp.h"
#include "cnet_ppp.h"

#if CNET_PPP
/*----------------------------------------------------------------------------*/
#define	AT_RESULT_OK			0
#define	AT_RESULT_CONNECT		1
#define	AT_RESULT_RING			2
#define	AT_RESULT_NOCARRIER		3
#define	AT_RESULT_ERROR			4
#define	AT_RESULT_REJECT		16
#define	AT_RESULT_DELAYED		29
#define	AT_RESULT_UNKNOWN		254
#define	AT_RESULT_TIMEOUT		255

#define	MODULE_STATE_INIT		0
#define	MODULE_STATE_NULL		1
#define	MODULE_STATE_CONNECTED	2
#define	MODULE_STATE_END		MODULE_STATE_INIT

static unsigned char			link_state		= MODULE_STATE_INIT;

#pragma ioreg
/*----------------------------------------------------------------------------*/
static short
recvtocrlf(char *out, short outlen)
{
	int				num		= 0;
	short			len		= 0;
	char			buf		= 0;

	if (out == (char *)0) {
		outlen = 0;
	}
	num = 10000;
	while (1) {
		while (uart1_recv((unsigned char *)&buf, 1) == 0) {
			if (num == 0) {
				len = -1;
				goto end;
			}
			num--;
			msleep(1);
		}
		if (buf == '\n') {
			break;
		}
		if (buf == '\r') {
			continue;
		}
		if (len < 0x7fff) {
			if (len < outlen) {
				out[len] = buf;
			}
			len++;
		}
	}

end:
	return len;
}
/*----------------------------------------------------------------------------*/
static unsigned char
recv_resulta(void)
{
	short			ret;
	char			buf[10]	= {0};

	/* Result Code */
	ret = recvtocrlf(buf, sizeof(buf));
	if (ret == 0) {
		ret = recvtocrlf(buf, sizeof(buf));
	}
	if (ret < 0) {
		ret = AT_RESULT_TIMEOUT;
		goto end;
	}
	switch ((int)ret) {
	case 2:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "OK", 2) == 0) {
			ret = AT_RESULT_OK;
		}
		break;
	case 5:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "ERROR", 5) == 0) {
			ret = AT_RESULT_ERROR;
		}
		break;
	case 4:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "RING", 4) == 0) {
			ret = AT_RESULT_RING;
		}
		break;
	case 6:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "REJECT", 6) == 0) {
			ret = AT_RESULT_REJECT;
		}
		break;
	case 7:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "CONNECT", 7) == 0) {
			ret = AT_RESULT_CONNECT;
		}
		else if (strncmp(buf, "DELAYED", 7) == 0) {
			ret = AT_RESULT_DELAYED;
		}
		break;
	case 10:
		ret = AT_RESULT_UNKNOWN;
		if (strncmp(buf, "NO CARRIER", 10) == 0) {
			ret = AT_RESULT_NOCARRIER;
		}
		break;
	default:
		ret = AT_RESULT_UNKNOWN;
	}

end:
	return (unsigned char)ret;
}
/*----------------------------------------------------------------------------*/
static unsigned char
at_command(const char *command, short len, char *out, short *outlen)
{
	unsigned char	ret;

	while (uart1_recv(&ret, 1) == 1) {
		__nop();
	}
	/* send command */
	uart1_send((const unsigned char *)"AT", 2);
	uart1_send((const unsigned char *)command, len);
	uart1_send((const unsigned char *)"\r\n", 2);
	/* receive echo */
	if (recvtocrlf((char *)0, 0) < 0) {
		ret = AT_RESULT_TIMEOUT;
		goto end;
	}
	/* receive information */
	if (outlen == (short *)0) {
		len = recvtocrlf((char *)0, 0);
	}
	else {
		len = recvtocrlf(out, *outlen);
		if (out == (char *)0) {
			*outlen = 0;
		}
		else {
			if (len < *outlen) {
				*outlen = len;
			}
		}
	}
	/* receive result code */
	ret = recv_resulta();

end:
	return ret;
}
/*----------------------------------------------------------------------------*/
int
m1xta01a_init(void)
{
	short			len;
	unsigned char	ret;
	char			buf	= 0;

	if (link_state != MODULE_STATE_INIT) {
		return -1;
	}

	/* open serial i/o */
	uart1_init(UART_SPEED_19200|UART_PARITY_NONE|UART_CHARLEN_8|UART_STOPBIT_1);
	uart1_option(BINARY_MODE|UART_FC_HARDWARE);
	uart1_recvstart();

	ret = at_command((const char *)0, 0, (char *)0, (short *)0);
	if (ret != AT_RESULT_OK) {
		uart1_end();
		goto end;
	}

	/* serial speed */
	ret = at_command("+IPR=115200", 11, (char *)0, (short *)0);
	if (ret != AT_RESULT_OK) {
		uart1_end();
		goto end;
	}

	/* reopen serial i/o */
	uart1_init(UART_SPEED_115200|UART_PARITY_NONE|UART_CHARLEN_8|UART_STOPBIT_1);
	uart1_option(BINARY_MODE|UART_FC_HARDWARE);
	uart1_recvstart();

	/* wait 2sec */
	msleep(2000);

	/* wireless speed */
	len = 1;
	ret = at_command("$50?", 4, &buf, &len);
	if (ret != AT_RESULT_OK) {
		uart1_end();
		goto end;
	}
	if (len == 1 && buf == '1') {
		ret = at_command("$50=0", 5, (char *)0, (short *)0);
		if (ret != AT_RESULT_OK) {
			uart1_end();
			goto end;
		}
	}

	link_state = MODULE_STATE_NULL;

end:
	return (int)ret;
}
/*----------------------------------------------------------------------------*/
int
m1xta01a_connect(void)
{
	unsigned char	ret;
	unsigned char	num;

	if (link_state != MODULE_STATE_NULL) {
		return -1;
	}

	/* connect */
	num = 5;
	do {
		ret = at_command("D#99", 4, (char *)0, (short *)0);
		if (ret == AT_RESULT_CONNECT) {
			ret = ppp_open(PPP_AUTH_TYPE, PPP_USER, PPP_PASS);
			if (ret != 0) {
				recv_resulta();
			}
			else {
				link_state = MODULE_STATE_CONNECTED;
			}
			break;
		}
		num--;
		msleep(1000);
	} while (num > 0);

	return (int)ret;
}
/*----------------------------------------------------------------------------*/
void
m1xta01a_disconnect(void)
{
	unsigned char	ret;

	if (link_state != MODULE_STATE_CONNECTED) {
		return;
	}

	ppp_close();

	while (uart1_recv(&ret, 1) == 1) {
		__nop();
	}

	/* disconnect request */
	P1.0 = 1;

	/* receive result code */
	ret = recv_resulta();

	/* enable serial i/o */
	P1.0 = 0;

	link_state = MODULE_STATE_NULL;
}
/*----------------------------------------------------------------------------*/
void
m1xta01a_end(void)
{
	unsigned char	ret;

	if (link_state == MODULE_STATE_INIT) {
		return;
	}
	if (link_state == MODULE_STATE_CONNECTED) {
		m1xta01a_disconnect();
	}

	/* shutdown */
	ret = at_command("$40=1", 5, (char *)0, (short *)0);

	/* wait 10sec */
	msleep(10000);

	/* close serial i/o */
	uart1_end();

	link_state = MODULE_STATE_END;
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_PPP */
