/*----------------------------------------------------------------------------*/
/* File Information                                                           */
/*----------------------------------------------------------------------------*/
/* Name       : uart_tk850_jg2.c                                              */
/* Type       : C Programming Language Source Code                            */
/* Version    : E2.00                                                         */
/* Date       : 2006 DEC 28                                                   */
/* CPU        : V850 Family                                                   */
/* About      : NEC Electronics V850 Lightweight Network Protocol Library     */
/*              UART API Source Code for TK-850/JG2+NET                       */
/*----------------------------------------------------------------------------*/
/* Copyright(C) NEC Electronics 2006                                          */
/* NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY                               */
/* All rights reserved by NEC Electronics Corporation.                        */
/* Use of copyright notice does not evidence publication                      */
/*----------------------------------------------------------------------------*/

#include "config.h"
#include "cnet_uart.h"

#if CNET_UART1
#if (CONFIG_LINKLAYER == 1)
#define UART1_BUFSIZE	8
#define UART1_RECVLIMIT	6
#elif (CONFIG_LINKLAYER == 2)
#define UART1_BUFSIZE	1504
#define UART1_RECVLIMIT	1500
#endif	/* CONFIG_LINKLAYER */
static int				wait1r					= 0;
static int				wait1s					= 0;
static unsigned short	uart1offset				= 0;
static unsigned short	uart1len				= 0;
static unsigned char	uart1buf[UART1_BUFSIZE]	= {0};
static unsigned char	uart1mode				= 0;
static unsigned char	uart1state				= 0;
#endif	/* CNET_UART1 */

#if CNET_UART2
#define UART2_BUFSIZE	8
static int				wait2r					= 0;
static int				wait2s					= 0;
static unsigned short	uart2offset				= 0;
static unsigned short	uart2len				= 0;
static unsigned char	uart2buf[UART2_BUFSIZE]	= {0};
static unsigned char	uart2mode				= 0;
static unsigned char	uart2state				= 0;
#endif	/* CNET_UART2 */

#pragma ioreg

#if CNET_UART1
/*----------------------------------------------------------------------------*/
void
uart1_send(const unsigned char *msg, unsigned short len)
{
	if (len <= 0) {
		return;
	}

	UA1TIF = 0;
	UA1TMK = 0;
	cnet_wait_set(&wait1s, 0);
	while (len > 0) {
		if (uart1mode & UART_FC_HARDWARE) {
			while (P0.6 != 0) {
				__nop();
			}
		}
		UA1TX = *msg;
		msg++;
		len--;
		cnet_wait(&wait1s);
	}
	UA1TMK = 1;
}
/*----------------------------------------------------------------------------*/
void
uart1_recvstart(void)
{
	char	tmp;

	tmp = UA1RX;
	UA1RIF = 0;
	UA1RMK = 0;
	UA1CTL0.5 = 1;
	uart1len = 0;
	uart1offset = 0;
	uart1state = 1;
	P3L.5 = 0;
}
/*----------------------------------------------------------------------------*/
void
uart1_recvstop(void)
{
	UA1RMK = 1;
	UA1CTL0.5 = 0;
//	UA1RIF = 0;
	uart1len = 0;
	uart1offset = 0;
	uart1state = 0;
	P3L.5 = 0;
}
/*----------------------------------------------------------------------------*/
static unsigned int
recv1_line(unsigned char *msg, unsigned short len, unsigned int num)
{
	UA1RMK = 1;
	while (uart1len > 0 && num < 0x80000000) {
		if (uart1mode & ECHO_BACK_MODE) {
			uart1_send(&uart1buf[uart1offset], 1);
		}
		if (uart1buf[uart1offset] == '\b') {
			if (num > 0) {
				num--;
			}
		}
		else {
			if (num < len) {
				msg[num] = uart1buf[uart1offset];
			}
			num++;
			if (uart1buf[uart1offset] == '\r') {
				if (uart1mode & ECHO_BACK_MODE) {
					uart1_send((const unsigned char *)"\n", 1);
				}
				num += 0x80000000;
			}
		}

		uart1offset++;
		if (uart1offset >= UART1_BUFSIZE) {
			uart1offset = 0;
		}
		uart1len--;
	}

	if (uart1len == 0) {
		uart1offset = 0;
	}
	if (uart1mode & UART_FC_HARDWARE) {
		if (uart1len < UART1_RECVLIMIT) {
				/* enable receive */
				P3L.5 = 0;
		}
	}
	UA1RMK = 0;

	return num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv1_line_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	unsigned int	num		= 0;

	cnet_wait_set(&wait1r, 0);
	while (num < 0x80000000) {
		if (uart1len == 0) {
			if (cnet_wait_timeout(&wait1r, timeout) <= 0) {
				break;
			}
		}
		num = recv1_line(msg, len, num);
	}
	wait1r = 0;
	num &= 0x7fffffff;
	if (num > len) {
		num = len;
	}

	return (unsigned short)num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv1_bin(unsigned char *msg, unsigned short len, unsigned short num)
{
	UA1RMK = 1;
	while (uart1len > 0 && num < len) {
		msg[num] = uart1buf[uart1offset];
		num++;
		uart1offset++;
		if (uart1offset >= UART1_BUFSIZE) {
			uart1offset = 0;
		}
		uart1len--;
	}
	if (uart1len == 0) {
		uart1offset = 0;
	}
	if (uart1mode & UART_FC_HARDWARE) {
		if (uart1len < UART1_RECVLIMIT) {
				/* enable receive */
				P3L.5 = 0;
		}
	}
	UA1RMK = 0;

end:
	return num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv1_bin_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	unsigned short	num	= 0;

	cnet_wait_set(&wait1r, 0);
	while (num < len) {
		if (uart1len == 0) {
			if (cnet_wait_timeout(&wait1r, timeout) <= 0) {
				break;
			}
		}
		num = recv1_bin(msg, len, num);
	}
	wait1r = 0;

	return num;
}
/*----------------------------------------------------------------------------*/
unsigned short
uart1_recv(unsigned char *msg, unsigned short len)
{
	if (uart1state == 0) {
		goto end;
	}
	if (msg == (unsigned char *)0) {
		len = 0;
	}

	if (uart1mode & (LINE_MODE | ECHO_BACK_MODE)) {
		unsigned int	num;

		num = recv1_line(msg, len, 0);
		num &= 0x7fffffff;
		if (num > len) {
			num = len;
		}
		return (unsigned short)num;
	}
	else {
		if (len > 0) {
			return recv1_bin(msg, len, 0);
		}
	}

end:
	return 0;
}
/*----------------------------------------------------------------------------*/
unsigned short
uart1_recv_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	if (uart1state == 0) {
		goto end;
	}
	if (msg == (unsigned char *)0) {
		len = 0;
	}

	if (uart1mode & (LINE_MODE | ECHO_BACK_MODE)) {
		return recv1_line_timeout(msg, len, timeout);
	}
	else {
		if (len > 0) {
			return recv1_bin_timeout(msg, len, timeout);
		}
	}

end:
	return 0;
}
/*----------------------------------------------------------------------------*/
void
uart1_end(void)
{
	if (uart1state == 1) {
		uart1_recvstop();
	}
	PM1.0 = 1;
	PM3L.5 = 1;

	while (UA1TSF);
	UA1CTL0.6 = 0;
	UA1CTL0.7 = 0;
}
/*----------------------------------------------------------------------------*/
void
uart1_init(unsigned char para)
{
	unsigned int	num;

	/* Initialize */
	uart1_end();

	num = para & 0x0f;
	if (num >= UART_SPEED_300 && num <= UART_SPEED_76800) {
		UA1CTL2 = 0x82;
		UA1CTL1 = (unsigned char)(0x08 - num);
	}
	else {
		if (num == UART_SPEED_31250) {
			UA1CTL2 = 0xA0;
			UA1CTL1 = 0x01;
		} else if (num == UART_SPEED_115200) {
			UA1CTL2 = 0x57;
			UA1CTL1 = 0x00;
		} else if (num == UART_SPEED_153600) {
			UA1CTL2 = 0x41;
			UA1CTL1 = 0x00;
		} else if (num == UART_SPEED_312500) {
			UA1CTL2 = 0x20;
			UA1CTL1 = 0x00;
		}
	}

	/* Parity bit, character bit, and stop bit setting */
	num = 1024;
	para >>= 4;
	para |= 0x90;
	UA1CTL0 = para;
	while (num > 0) {
		__nop();
		num--;
	}
	para |= 0x40;
	UA1CTL0 = para;

	/* enable serial i/o */
	PM1.0 = 0;
	P1.0 = 0;
	/* enable receive */
	PM3L.5 = 0;
	P3L.5 = 0;

	/* wait */
	num = 256;
	while (num > 0) {
		__nop();
		num--;
	}
}
/*----------------------------------------------------------------------------*/
void 
uart1_option(unsigned char mode)
{
	uart1mode = mode;
}
/*----------------------------------------------------------------------------*/
void
uart1_recv_interrupt(void)
{
	int		num;

	if (uart1len < UART1_BUFSIZE) {
		num = uart1offset + uart1len;
		if (num >= UART1_BUFSIZE) {
			num -= UART1_BUFSIZE;
		}
		uart1buf[num] = UA1RX;
		uart1len++;
		if (wait1r != 0) {
			cnet_ready(&wait1r);
		}
	}
	if (uart1mode & UART_FC_HARDWARE) {
		if (uart1len >= UART1_RECVLIMIT) {
				/* disable receive */
				P3L.5 = 1;
		}
	}
}
/*----------------------------------------------------------------------------*/
void
uart1_send_interrupt(void)
{
	cnet_ready(&wait1s);
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_UART1 */
#if CNET_UART2
/*----------------------------------------------------------------------------*/
void
uart2_send(const unsigned char *msg, unsigned short len)
{
	if (len <= 0) {
		return;
	}

	UA2TIF = 0;
	UA2TMK = 0;
	cnet_wait_set(&wait2s, 0);
	while (len > 0) {
		UA2TX = *msg;
		msg++;
		len--;
		cnet_wait(&wait2s);
	}
	UA2TMK = 1;
}
/*----------------------------------------------------------------------------*/
void
uart2_recvstart(void)
{
	char	tmp;

	tmp = UA2RX;
	UA2RIF = 0;
	UA2RMK = 0;
	UA2CTL0.5 = 1;
	uart2len = 0;
	uart2offset = 0;
	uart2state = 1;
}
/*----------------------------------------------------------------------------*/
void
uart2_recvstop(void)
{
	UA2RMK = 1;
	UA2CTL0.5 = 0;
//	UA2RIF = 0;
	uart2len = 0;
	uart2offset = 0;
	uart2state = 0;
}
/*----------------------------------------------------------------------------*/
static unsigned int
recv2_line(unsigned char *msg, unsigned short len, unsigned int num)
{
	UA2RMK = 1;
	while (uart2len > 0 && num < 0x80000000) {
		if (uart2mode & ECHO_BACK_MODE) {
			uart2_send(&uart2buf[uart2offset], 1);
		}
		if (uart2buf[uart2offset] == '\b') {
			if (num > 0) {
				num--;
			}
		}
		else {
			if (num < len) {
				msg[num] = uart2buf[uart2offset];
			}
			num++;
			if (uart2buf[uart2offset] == '\r') {
				if (uart2mode & ECHO_BACK_MODE) {
					uart2_send((const unsigned char *)"\n", 1);
				}
				num += 0x80000000;
			}
		}

		uart2offset++;
		if (uart2offset >= UART2_BUFSIZE) {
			uart2offset = 0;
		}
		uart2len--;
	}

	if (uart2len == 0) {
		uart2offset = 0;
	}
	UA2RMK = 0;

	return num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv2_line_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	unsigned int	num		= 0;

	cnet_wait_set(&wait2r, 0);
	while (num < 0x80000000) {
		if (uart2len == 0) {
			if (cnet_wait_timeout(&wait2r, timeout) <= 0) {
				break;
			}
		}
		num = recv2_line(msg, len, num);
	}
	wait2r = 0;
	num &= 0x7fffffff;
	if (num > len) {
		num = len;
	}

	return (unsigned short)num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv2_bin(unsigned char *msg, unsigned short len, unsigned short num)
{
	UA2RMK = 1;
	while (uart2len > 0 && num < len) {
		msg[num] = uart2buf[uart2offset];
		num++;
		uart2offset++;
		if (uart2offset >= UART2_BUFSIZE) {
			uart2offset = 0;
		}
		uart2len--;
	}
	if (uart2len == 0) {
		uart2offset = 0;
	}
	UA2RMK = 0;

end:
	return num;
}
/*----------------------------------------------------------------------------*/
static unsigned short
recv2_bin_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	unsigned short	num	= 0;

	cnet_wait_set(&wait2r, 0);
	while (num < len) {
		if (uart2len == 0) {
			if (cnet_wait_timeout(&wait2r, timeout) <= 0) {
				break;
			}
		}
		num = recv2_bin(msg, len, num);
	}
	wait2r = 0;

	return num;
}
/*----------------------------------------------------------------------------*/
unsigned short
uart2_recv(unsigned char *msg, unsigned short len)
{
	if (uart2state == 0) {
		goto end;
	}
	if (msg == (unsigned char *)0) {
		len = 0;
	}

	if (uart2mode & (LINE_MODE | ECHO_BACK_MODE)) {
		unsigned int	num;

		num = recv2_line(msg, len, 0);
		num &= 0x7fffffff;
		if (num > len) {
			num = len;
		}
		return (unsigned short)num;
	}
	else {
		if (len > 0) {
			return recv2_bin(msg, len, 0);
		}
	}

end:
	return 0;
}
/*----------------------------------------------------------------------------*/
unsigned short
uart2_recv_timeout(unsigned char *msg, unsigned short len, unsigned long timeout)
{
	if (uart2state == 0) {
		goto end;
	}
	if (msg == (unsigned char *)0) {
		len = 0;
	}

	if (uart2mode & (LINE_MODE | ECHO_BACK_MODE)) {
		return recv2_line_timeout(msg, len, timeout);
	}
	else {
		if (len > 0) {
			return recv2_bin_timeout(msg, len, timeout);
		}
	}

end:
	return 0;
}
/*----------------------------------------------------------------------------*/
void
uart2_end(void)
{
	if (uart2state == 1) {
		uart2_recvstop();
	}

	while (UA2TSF);
	UA2CTL0.6 = 0;
	UA2CTL0.7 = 0;
}
/*----------------------------------------------------------------------------*/
void
uart2_init(unsigned char para)
{
	unsigned int	num;

	/* Initialize */
	uart2_end();

	num = para & 0x0f;
	if (num >= UART_SPEED_300 && num <= UART_SPEED_76800) {
		UA2CTL2 = 0x82;
		UA2CTL1 = (unsigned char)(0x08 - num);
	}
	else {
		if (num == UART_SPEED_31250) {
			UA2CTL2 = 0xA0;
			UA2CTL1 = 0x01;
		} else if (num == UART_SPEED_115200) {
			UA2CTL2 = 0x57;
			UA2CTL1 = 0x00;
		} else if (num == UART_SPEED_153600) {
			UA2CTL2 = 0x41;
			UA2CTL1 = 0x00;
		} else if (num == UART_SPEED_312500) {
			UA2CTL2 = 0x20;
			UA2CTL1 = 0x00;
		}
	}

	/* Parity bit, character bit, and stop bit setting */
	num = 1024;
	para >>= 4;
	para |= 0x90;
	UA2CTL0 = para;
	while (num > 0) {
		__nop();
		num--;
	}
	para |= 0x40;
	UA2CTL0 = para;

	/* wait */
	num = 256;
	while (num > 0) {
		__nop();
		num--;
	}
}
/*----------------------------------------------------------------------------*/
void 
uart2_option(unsigned char mode)
{
	uart2mode = mode;
}
/*----------------------------------------------------------------------------*/
void
uart2_recv_interrupt(void)
{
	int		num;

	if (uart2len < UART2_BUFSIZE) {
		num = uart2offset + uart2len;
		if (num >= UART2_BUFSIZE) {
			num -= UART2_BUFSIZE;
		}
		uart2buf[num] = UA2RX;
		uart2len++;
		if (wait2r != 0) {
			cnet_ready(&wait2r);
		}
	}
}
/*----------------------------------------------------------------------------*/
void
uart2_send_interrupt(void)
{
	cnet_ready(&wait2s);
}
/*----------------------------------------------------------------------------*/
#endif	/* CNET_UART2 */
